/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Util;

public class VariantenTalentgruppe implements Varianten {

    /** Repraesentiert die 'Talentgruppe' der Nahkampffertigkeiten. */
    public static final String NAHKAMPF =
        "VariantenTalentgruppe.Nahkampffertigkeiten";

    /** Repraesentiert die 'Talentgruppe' der Fernkampffertigkeiten. */
    public static final String FERNKAMPF =
        "VariantenTalentgruppe.Fernkampffertigkeiten";

    /**
     * Die Liste der Talentgruppen und die beiden Strings
     * {@link #NAHKAMPF} und {@link #FERNKAMPF}.
     */
    private final List< Variante > varianten;

    VariantenTalentgruppe() {

        varianten = new ArrayList< Variante >( 10 );

        varianten.add( new Variante( Talentgruppe.GESELLSCHAFT ) );
        varianten.add( new Variante( Talentgruppe.HANDWERK ) );
        varianten.add( new Variante( Talentgruppe.KAMPF ) );
        varianten.add( new Variante( NAHKAMPF ) );
        varianten.add( new Variante( FERNKAMPF ) );
        varianten.add( new Variante( Talentgruppe.KOERPER ) );
        varianten.add( new Variante( Talentgruppe.SCHRIFTEN ) );
        varianten.add( new Variante( Talentgruppe.SPRACHEN ) );
        varianten.add( new Variante( Talentgruppe.NATUR ) );
        varianten.add( new Variante( Talentgruppe.WISSEN ) );

        Collections.sort( varianten, new Comparator<Object>() {

            public int compare( final Object o1, final Object o2 ) {

                return Util.compare( o1.toString(), o2.toString() );
            }
        });
    }

    /**
     * @throws UnsupportedOperationException wenn die Methode aufgerufen wird.
     */
    public void einfuegen( final Variante variante ) {
        throw new UnsupportedOperationException();
    }

    /**
     * Die Variante die zum uebergebenen objekt passt.
     */
    public Variante getVariante( final Object objekt ) {
        
        if ( objekt instanceof ElementMitName ) {
            
            return new Variante( (ElementMitName) objekt );
        
        } else if ( objekt instanceof String ) { 
        
            return new Variante( (String) objekt );
            
        } else {
            
            return (Variante) objekt;
        }
    }
    
    /**
     *
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * @return eine Liste der moeglichen Talentgruppen, die Liste enthaelt auch
     *         die beiden Strings {@link #NAHKAMPF} und
     *         {@link #FERNKAMPF}.
     */
    public List< Variante > getVarianten(final Held held) {
        return new ArrayList< Variante >( varianten );
    }

    /** @return {@code true} */
    public boolean hatVarianten() {
        return true;
    }

    /** @return {@code false} */
    public boolean istFreieVariante() {
        return false;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return "Talentgruppen: " + getVarianten( null ).toString();
    }
}
