/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_BESITZ;
import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.i18n.I18n;

public class BesondererBesitz {

    /**
     * Der Schluessel fuer einen Text aus der Lokalisierungsdatei, falls es dort
     * einen Text gibt, oder die Beschreibung des Besitzes.
     */
    private final String id;

    /**
     * @param dieId
     *            Ein Schluessel fuer einen Text aus der Lokalisierungsdatei
     *            oder die Beschreibung des Besonderen Besitzes.
     */
    public BesondererBesitz( final String dieId ) {

        id = dieId;

    }

    /**
     * @return der Schluessel fuer einen Text aus der Lokalisierungsdatei oder
     *         ein Text fuer den Besonderen Besitz.
     */
    public String getId() {
        return id;
    }

    /**
     * @return die Beschreibung des besonderen Besitzes.
     */
    public final String getText() {

        if ( I18n.hatLokalisierungstext( id ) )  {

            return I18n.i18n( id );

        } else {

            return id;
        }
    }

    /**
     * @return ein XML-Element, das den Besitz repraesentiert.
     */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_BESITZ );

        element.setContent( id );

        return element;
    }
}
