/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_SONDERFERTIGKEIT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_TALENT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_SONDERFERTIGKEIT;

import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;

public class VerbilligteSonderfertigkeitStandard implements
        VerbilligteSonderfertigkeit {

    /**
     * Die Sonderfertigkeiten die zur Auswahl stehen.
     * Normalerweise nur gibt es nur eine.
     */
    private final Sonderfertigkeit sonderfertigkeit;

    /**
     * @param dieSonderfertigkeit die verbilligt wird.
     */
    public VerbilligteSonderfertigkeitStandard(
            final Sonderfertigkeit dieSonderfertigkeit ) {

        sonderfertigkeit = dieSonderfertigkeit;
    }

    public double getGpKosten() {
        return sonderfertigkeit.getApKosten() / 250.0;
    }

    /** @return die Sonderfertigkeit die verbilligt wird. */
    public final Sonderfertigkeit getSonderfertigkeit() {
        return sonderfertigkeit;
    }

    /**
     * @throws UnsupportedOperationException wenn die Methode aufgerufen wird.
     */
    public final List<Sonderfertigkeit> getSonderfertigkeiten( final Held held ) {
        throw new UnsupportedOperationException( String.format(
                "Es stehen nicht mehrere Sonderfertigkeiten zur Auswahl. Es " +
                "wird nur die Sonderfertigkeit %1$s verbilligt",
                sonderfertigkeit.getName() ) );
    }

    /** @return {@code true} */
    public final boolean hatAuswahl() {
        return false;
    }

    /**
     * @return der Name der Sonderfertigkeit, die verbilligt wird.
     */
    @Override
    public final String toString() {
        return sonderfertigkeit.getName();
    }

    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_SONDERFERTIGKEIT );

        element.setAttribute( ATTR_SONDERFERTIGKEIT, sonderfertigkeit.getId() );

        if ( sonderfertigkeit.hatTalent() ) {

            element.setAttribute( ATTR_TALENT,
                                  sonderfertigkeit.getTalent().getId() );
        }

        return element;
    }
}
