/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten;

import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

public interface TalentListe {

    /** Eine Talent-Liste die nur aus den Fremdsprache eines Helden besteht. */
    public static final TalentListe FREMDSPRACHEN =
        new TalentListeFremdsprachen();

    /** Eine Talent-Liste die nur aus der Muttersprache eines Helden besteht. */
    public static final TalentListe MUTTERSPRACHE =
        new TalentListeMuttersprache();

    /**
     * Eine Talent-Liste die aus Garethi und Tulamydia besteht. Ist eine der
     * Sprachen Muttersprache kommt sie in der Liste nicht vor.
     */
    public static final TalentListe NICHT_MUTTERSPRACHE =
        new TalentListeNichtMuttersprache();

    /**
     * Eine Talent-Liste die aus den Schriften der Muttersprache eines Helden
     * besteht.
     */
    public static final TalentListe SCHRIFTEN_MUTTERSPRACHE =
        new TalentListeSchriftMuttersprache();

    /**
     * Liefert eine Beschreibung der Talent-Liste.
     * Das kann ein einfache Aufzaehlung aller Talente in der Liste sein,
     * z.B. 'Singen, Tanzen oder Zechen', oder ein Umschreibung wie
     * 'Eine Fremdsprache' oder 'Die Schrift der Muttersprache'.
     *
     * @return eine Beschreibung der Talent-Liste.
     */
    public String getBeschreibung();

    /**
     * Diese Methode ist nur zur Berechnung der GP-Kosten die einem Baustein
     * durch einen Talentbonus entstehen.
     *
     * @return eine Liste von Talenten, zur Berechnung der GP-Kosten.
     */
    public List< Talent > getTalente();

    /**
     * @param held fuer den die Talentliste sein soll.
     * @return die zu {@code held} passende Liste an Talenten.
     */
    public List< Talent > getTalente( final Held held );

    /**
     * @return die Talentgruppe der dieser Bonus zugeordnet ist.
     */
    public Talentgruppe getTalentgruppe();

    /**
     * @return Die XML-Repraesentation der Liste.
     */
    public IXMLElement toXml();
}
