/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.bausteine.moeglicheprof;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_ALLE_VON;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_PROFESSION;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_PROFESSIONEN;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;
import de.bernhardjung.dsaprogramm.bausteine.Profession;
import de.bernhardjung.dsaprogramm.util.Strings;

public class MoeglicheProfessionenAlleVon implements MoeglicheProfessionen {

    /** Die moeglichen Professionen. */
    private final Collection< Profession > professionen;

    /**
     * @param dieProfessionen die moegliche sind.
     */
    public MoeglicheProfessionenAlleVon(
            final Collection< Profession > dieProfessionen ) {

        professionen = new HashSet< Profession >( dieProfessionen );
    }

    /** {@inheritDoc} */
    public Collection< Profession > getProfessionen() {

        final List< Profession > liste =
            new ArrayList< Profession >( professionen );

        Collections.sort( liste, Profession.NAME_COMPARATOR );

        return liste;
    }

    /** {@inheritDoc} */
    public boolean istMoeglich(final Profession profession) {
        return professionen.contains( profession );
    }

    /** @return eine Beschreibung welche Professionen moegliche sind. */
    @Override
    public String toString() {

        final List< Profession > liste =
            new ArrayList< Profession >( professionen );

        Collections.sort( liste, Profession.NAME_COMPARATOR );

        return Strings.erzeugeAufzaehlungMitUnd( liste );
    }

    /** {@inheritDoc} */
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_PROFESSIONEN );

        final IXMLElement alleVonElement = new XMLElement( TAG_ALLE_VON );

        for ( final Profession profession : professionen ) {

            final IXMLElement profElement = new XMLElement( TAG_PROFESSION );

            profElement.setContent( profession.getId() );

            alleVonElement.addChild( profElement );
        }

        element.addChild( alleVonElement );

        return element;
    }

}
