/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_GELAENDEKUNDE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_SONDERFERTIGKEIT;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_VERB_SONDERFERTIGKEITEN;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeit;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeitStandard;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;

public class FactoryVerbilligteSonderfertigkeitenIn
extends FactorySonderfertigkeitIn {

    public final List< VerbilligteSonderfertigkeit > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< VerbilligteSonderfertigkeit > verbilligte;
        verbilligte = new ArrayList< VerbilligteSonderfertigkeit >();

        final IXMLElement verbilligteElement =
            wurzelElement.getFirstChildNamed( TAG_VERB_SONDERFERTIGKEITEN );

        if ( null != verbilligteElement ) {

            for ( final IXMLElement verbilligt :
                                    verbilligteElement.getChildren() ) {

                verbilligte.add( erzeugeVerbilligte( verbilligt ) );

            }
        }

        return verbilligte;
    }

    private VerbilligteSonderfertigkeit erzeugeVerbilligte(
            final IXMLElement verbilligtElement ) {

        final String name = verbilligtElement.getName();

        if ( name.equals( TAG_SONDERFERTIGKEIT ) ) {

            final Sonderfertigkeit sonderfertigkeit =
                ermitteleSonderfertigkeit( verbilligtElement );

            return new VerbilligteSonderfertigkeitStandard( sonderfertigkeit );

        } else if ( name.equals( TAG_GELAENDEKUNDE ) ){

            return VerbilligteSonderfertigkeit.GELAENDEKUNDE;

        } else {

            throw new IllegalArgumentException( String.format(
                    "Das Tag %1$s fuer eine verbilligte Sonderfertigkeit ist " +
                    "unbekannt.", name ) );
        }
    }
}
