/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.daten.factorys.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BESCHREIBUNG;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VARIANTE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_VERBILLIGTE_VARIANTEN;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteVariante;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;

public class FactoryVerbilligteVariantenIn extends FactorySonderfertigkeitIn {

    /**
     * @param wurzelElement des XML-Dokuments.
     * @return die Sonderfertigkeiten bei denen eine Variante verbilligt wird.
     */
    public final List< VerbilligteVariante > erzeugen(
            final IXMLElement wurzelElement ) {

        final List< VerbilligteVariante > liste;
        liste = new ArrayList<VerbilligteVariante>();

        final IXMLElement verbilligteElement =
            wurzelElement.getFirstChildNamed( TAG_VERBILLIGTE_VARIANTEN );

        if ( null != verbilligteElement ) {

            for ( final IXMLElement variante :
                        verbilligteElement.getChildren() ) {

                // Erst die Sonderfertigkeit ermitteln.
                // ************************************
                final Sonderfertigkeit sonderfertigkeit =
                    ermitteleSonderfertigkeit( variante );

                // Jetzt die Variante bestimmen, wenn eine da ist.
                // ***********************************************
                final String varianteString =
                    variante.getAttribute( ATTR_VARIANTE );

                final Variante varianteObjekt;
                
                if ( null == varianteString )  {
                    
                    varianteObjekt = null;
                
                } else {
                    
                    varianteObjekt = new Variante( varianteString );
                }
                
                // Als naechstes Pruefen ob es einen Beschreibungstext gibt.
                // *********************************************************
                final String beschreibung;
                beschreibung = variante.getAttribute( ATTR_BESCHREIBUNG );

                // Fertig.
                // *******
                liste.add( new VerbilligteVariante( 
                        sonderfertigkeit,
                        varianteObjekt,
                        beschreibung ) );
            }
        }

        return liste;
    }
}
