package de.bernhardjung.dsaprogramm.gui;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Hier kann der Benutzer die Daten fuer einen benutzerdefinierten
 * Vor-/Nachteil eingeben.
 */
public class PanelBenutzerdefinierterVorNachteil extends JPanel implements Uebersetzbar {

    /** Steht als Text zwischen den Spinnern fuer Min- und Max-Stufe. */
    private static final String BIS = "PanelBenutzerdefinierterVorNachteil.Bis";
    /** Label fuer den Spinner mit den GP-Kosten. */
    private static final String GPKOSTEN = "PanelBenutzerdefinierterVorNachteil.GpKosten";
    /** Label fuer den Radiobutton mit dem man einen Nachteil eingibt. */
    private static final String NACHTEIL = "PanelBenutzerdefinierterVorNachteil.Nachteil";
    /** Label fuer das Textfeld in das man den Namen des Vor-/Nachteils eingibt. */
    private static final String NAME = "PanelBenutzerdefinierterVorNachteil.Name";
    /** Gibt an, dass die GP-Kosten nur pro diese Stufe zu entrichten sind. */
    private static final String PRO_STUFE = "PanelBenutzerdefinierterVorNachteil.ProStufe";
    /** Label fuer den Spinner mit der Schrittweite fuer die Stufe */
    private static final String SCHRITT = "PanelBenutzerdefinierterVorNachteil.Schritt";
    /** Label die Checkbox mit der man angibt, dass der Vor-/Nachteil Stufen hat. */
    private static final String STUFEN = "PanelBenutzerdefinierterVorNachteil.Stufen";
    /** Label vor den Radiobuttons mit denen man zwische Vor- und Nachteil wählt. */
    private static final String TYP = "PanelBenutzerdefinierterVorNachteil.Typ";
    /** Label fur den Radiobutton mit dem man angibt, dass es sich ume einen Vorteil handelt.*/
    private static final String VORTEIL = "PanelBenutzerdefinierterVorNachteil.Vorteil";

    /** Spinner fuer die GP-Kosten. */
    private final JSpinner gpKosten;

    /** Checkbox mit der man angibt, ob der Vor-/Nachteil Stufen hat. */
    private final JCheckBox hatStufen;

    /** Gibt die maximal erlaubte Stufe an. */
    private final JSpinner hoechststufe;

    /** Gibt die kleinste erlaubte Stufe han. */
    private final JSpinner mindeststufe;

    /** Gibt an, dass man eine Nachteil erzeugen will. */
    private final JRadioButton nachteil;

    /** Hier gibt man den Namen des Vor-/Nachteils ein. */
    private final JTextField name;

    /** Die GP-Kosten sind pro diese Stufen zu entrichten. */
    private final JSpinner proStufe;

    /** Die Schrittweite mit der die Stufe angehoben/gesenkt wird. */
    private final JSpinner schrittweite;

    /** Kann die Texte im Panel beim Sprachwechsel neu uebersetzen. */
    private final Uebersetzer uebersetzer;

    /** Gibt an, dass man eine Vorteil erschaffen will. */
    private final JRadioButton vorteil;

    /** Erzeugt alle Elemente (Spinner,Buttons,...) der Benutzeroberflaeche. */
    public PanelBenutzerdefinierterVorNachteil() {

        uebersetzer = new Uebersetzer();

        gpKosten = Util.erzeugeSpinner( 1, 1, 99, 1 );

        hatStufen = new JCheckBox();

        hoechststufe = Util.erzeugeSpinner( 1, 1, 99, 1 );
        hoechststufe.setEnabled( false );
        hoechststufe.addChangeListener( new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                int min = ( Integer ) mindeststufe.getValue();
                int max = ( Integer ) hoechststufe.getValue();
                if ( min > max )
                    mindeststufe.setValue( max );
            }
        } );

        mindeststufe = Util.erzeugeSpinner( 1, 1, 99, 1 );
        mindeststufe.setEnabled( false );
        mindeststufe.addChangeListener( new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                int min = (Integer) mindeststufe.getValue();
                int max = (Integer) hoechststufe.getValue();
                if ( min > max )
                    hoechststufe.setValue( min );
            }
        } );

        nachteil = uebersetzer.erzeugeRadioButton( NACHTEIL );

        name = new JTextField();

        proStufe = Util.erzeugeSpinner( 1, 1, 99, 1 );
        proStufe.setEnabled( false );

        schrittweite = Util.erzeugeSpinner( 1, 1, 99, 1 );
        schrittweite.setEnabled( false );

        vorteil = uebersetzer.erzeugeRadioButton( VORTEIL );
        vorteil.setSelected( true );

        hatStufen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                mindeststufe.setEnabled( hatStufen.isSelected() );
                hoechststufe.setEnabled( hatStufen.isSelected() );
                schrittweite.setEnabled( hatStufen.isSelected() );
                proStufe.setEnabled( hatStufen.isSelected() );
            }
        } );

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add( vorteil );
        buttonGroup.add( nachteil );

        aufbauenBenutzeroberflaeche();
    }

    /** Erzeugt das Layout und baut die Benutzeroberflaeche auf. */
    private void aufbauenBenutzeroberflaeche() {

        FormLayout layout = new FormLayout( "P, 4dlu, P" );
        DefaultFormBuilder builder = new DefaultFormBuilder( layout, this );
        builder.setDefaultDialogBorder();

        builder.append( uebersetzer.erzeugeLabel( NAME ) );
        builder.append( name );

        builder.append( uebersetzer.erzeugeLabel( TYP ) );
        builder.append( erzeugeTypPanel() );

        builder.append( uebersetzer.erzeugeLabel( STUFEN ) );
        builder.append( erzeugeStufenPanel() );

        builder.append( uebersetzer.erzeugeLabel( GPKOSTEN ) );
        builder.append( erzeugeKostenPanel() );
    }

    /** Erzeugt das Sub-Panel in dem man die GP-Kosten waehlt. */
    private JPanel erzeugeKostenPanel() {

        FormLayout layout = new FormLayout( "P, 2dlu, P, 2dlu, P", "P" );
        CellConstraints cc = new CellConstraints();
        JPanel panel = new JPanel( layout );
        panel.add( gpKosten, cc.xy( 1, 1 ) );
        panel.add( uebersetzer.erzeugeLabel( PRO_STUFE ), cc.xy( 3, 1 ) );
        panel.add( proStufe, cc.xy( 5, 1 ) );
        return panel;
    }

    /** Erzeugt das Sub-Panel fuer die eingabe der Stufe. */
    private JPanel erzeugeStufenPanel() {

        FormLayout layout = new FormLayout( "P, 2dlu, P, 2dlu, P, 2dlu, P, 2dlu, P, 2dlu, P", "P" );
        CellConstraints cc = new CellConstraints();
        JPanel panel = new JPanel( layout );
        panel.add( hatStufen, cc.xy( 1, 1 ) );
        panel.add( mindeststufe, cc.xy( 3, 1 ) );
        panel.add( uebersetzer.erzeugeLabel( BIS ), cc.xy( 5, 1 ) );
        panel.add( hoechststufe, cc.xy( 7, 1 ) );
        panel.add( uebersetzer.erzeugeLabel( SCHRITT ), cc.xy( 9, 1 ) );
        panel.add( schrittweite, cc.xy( 11, 1 ) );
        return panel;
    }

    /** Erzeut das Subpanel in dem man zwischen Vor- und Nachteil waehlt. */
    private JPanel erzeugeTypPanel() {

        FormLayout layout = new FormLayout( "P, 2dlu, P", "P" );
        CellConstraints cc = new CellConstraints();
        JPanel panel = new JPanel( layout );
        panel.add( vorteil, cc.xy( 1, 1 ) );
        panel.add( nachteil, cc.xy( 3, 1 ) );
        return panel;
    }

    /** Liefert die eingegebenen Daten. */
    private Variante getVariante() {

        String nameString = name.getText().trim();
        Variante variante = new Variante( nameString );
        if (nachteil.isSelected())
            variante.setGpKosten( -(Integer) gpKosten.getValue() );
        else
            variante.setGpKosten( (Integer) gpKosten.getValue() );
        if ( hatStufen.isSelected() ) {
            variante.setHatStufen( true );
            variante.setMindeststufe( (Integer) mindeststufe.getValue() );
            variante.setHoechststufe( ( Integer ) hoechststufe.getValue() );
            variante.setSchrittweite( (Integer) schrittweite.getValue() );
            variante.setProStufe( (Integer) proStufe.getValue() );
        }
        return variante;
    }

    /**
     * Erzeugt einen Wert mit den eingegebenen Daten.
     * Der Wert kann dem helden hinzugefuegt werden.
     */
    public VorNachteilWert getWert() {

        VorNachteil vorNachteil;
        if (nachteil.isSelected())
            vorNachteil = VorNachteil.BENUTZERDEFINIERTER_NACHTEIL;
        else
            vorNachteil = VorNachteil.BENUTZERDEFINIERTER_VORTEIL;

        Variante variante = getVariante();

        int stufe;
        if (hatStufen.isSelected())
            stufe = (Integer) mindeststufe.getValue();
        else
            stufe = 0;

        VorNachteilWert wert = new VorNachteilWert( vorNachteil, variante, false );
        wert.setStufe( stufe );
        return wert;
    }

    /** Ubersetzt das Panel bei einem Sprachwechsel. */
    public void neuUebersetzen() {
        uebersetzer.neuUebersetzen();
    }
}
