/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFest;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFrei;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListe;
import de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten.TalentListeFest;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.erschaffung.Erschaffung;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.gui.bonus.*;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

public class PanelErschaffung 
extends WechselbaresPanel 
implements Uebersetzbar {

    /**
     * Die Frage die dem Benutzer gestellt wird, wenn er die Erschaffung
     * abbrechen will.
     */
    private static final String ABBRECHEN_FRAGE = "PanelErschaffung.ErschaffungAbbrechen.Frage";

    /**
     * Erklaert dem Benutzer, dass er zu einem Wert in rot, über einen Tooltip,
     * nähere Informationen erhalten kann.
     */
    private static final String HINWEIS = "PanelErschaffung.Hinweis";

    /**
     * Der Knopf mit dem man zum naechsten Schritt der Erschaffung gelangen
     * kann.
     */
    private final JButton knopfWeiter;

    /**
     * Der Manager fuer die aktuelle Charaktererschaffung.
     */
    private ErschaffungManager manager;
    
    /**
     * Erklaert dem Benutzer, dass er bei einem rot geschriebenen Text einen
     * Tooltip findet.
     */
    private final JLabel hinweis;

    /** Die Benutzerschnittstelle, die aktuell im Panel angezeigt wird. */
    private ResetbaresPanel aktuellesPanel;

    /**
     * Das Panel auf dem der Benutzer aus Bonis wahlen darf, die er z.B. durch
     * die Rasse erhalten hat. 
     */
    private PanelBonusAnzeige panelBonusAnzeige;
    
    /** Das Panel auf dem die verbliebenen AP und GP angezeigt werden. */
    private PanelGp panelGp;

    /**
     * Das Panel mit dem Knopf, mit dem man bei der Erschaffung zum naechsten
     * Schritt ueber geht.
     */
    private JPanel panelKnopfWeiter;

    /**
     * Das Panel mit dem die zusaetzlichen Leitzauber fuer einen elfischen
     * Helden gewaehlt werden koennen.
     */
    private PanelLeitzauber panelLeitzauber;

    /** Das Panel um die Rasse, Kultur und Profession zu waehlen. */
    private PanelRasKulProf panelRasKulProf;

    /**
     * Ein Panel mit dem man eine Sonderfertigkeit aus einer Liste waehlen kann.
     */
    private PanelAuswahl< SonderfertigkeitWert > panelSonderfertigkeitWert;

    /**
     * Ein Panel mit dem man eine Sonderfertigkeit aus einer Liste waehlen kann.
     */
    private PanelAuswahl< Sonderfertigkeit > panelSonderfertigkeit;

    /**
     * Erlaubt es eine Sprache aus einer Liste zu waehlen. Wird genutzt, um die
     * Zweitsprache der zwergischen Kultur zu waehlen.
     */
    private PanelAuswahl< Sprache > panelSprache;

    /**
     * Das Panel zum steigern des Helden nachdem Rasse, Kultur und Profession
     * gewaehlt worden sind.
     */
    private PanelSteigernErschaffung panelSteigern;

    /**
     * Das Panel mit dem eine Auswahl fuer einen Bonus wie 'Singen, Tanzen oder
     * Zechen +5' getroffen werden kann.
     */
    private PanelTalentBonusFest panelTalentBonusFest;

    /**
     * Das Panel mit dem eine Auswahl fuer den speziellen Bonus, den die
     * Profession Entdecker vergibt, getroffen werden kann.
     */
    private PanelTalentBonusFestEntdecker2 panelTalentBonusEntdecker;

    /**
     * Das Panel mit dem eine Auswahl fuer einen Bonus wie '4 Punkte auf
     * Oloarkh, Ologhaijan und/oder Thorwalsch verteilen' getroffen werden kann.
     */
    private PanelTalentBonusFrei panelTalentBonusFrei;

    /**
     * Das Panel mit dem man einen Vor-/Nachteil fuer den Helden waehlen kann.
     */
    private PanelAuswahl< VorNachteilWert > panelVorNachteilWert;

    /** Uebersetzt die angezeigten Texte. Den einen :-). */
    private final Uebersetzer uebersetzer;

    public PanelErschaffung() {

        super( new BorderLayout() );

        uebersetzer = new Uebersetzer();

        hinweis = uebersetzer.erzeugeLabel( HINWEIS );

        knopfWeiter = new JButton();

        add( getPanelGp(), BorderLayout.PAGE_START );

        add( getPanelKnopfWeiter(), BorderLayout.PAGE_END );
    }

    public final void anzeigen( final PanelBonus panel ) {
        
        final PanelBonusAnzeige bonusAnzeige = getPanelBonusAnzeige();
        
        bonusAnzeige.anzeigen( panel );
        
        if ( null != manager ) {
        
            bonusAnzeige.setOffeneBoni( manager.getOffeneBoni() );
        }
        
        anzeigen( bonusAnzeige );
    }
    
    /**
     * Bestimmt was im Zentrum des Hauptfensters dargestellt wird.
     * 
     * @param panel
     *            das dargestellt wird.
     */
    public final void anzeigen( final ResetbaresPanel panel ) {

        if ( null != panel && panel != aktuellesPanel ) {

            if ( null != aktuellesPanel ) {

                remove( aktuellesPanel );

            }

            aktuellesPanel = panel;

            aktuellesPanel.reset();

            add( aktuellesPanel, BorderLayout.CENTER );

            validate();
            repaint();
        }
    }

    public void erzeugeAllePanel( final Hauptpanel hauptpanel) {

        debugErzeugePanelGp();

        getPanelKnopfWeiter();

        debugErzeugePanelLeitzauber();

        debugErzeugePanelRasKulProf();

        debugErzeugePanelSonderfertigkeit();

        debugErzeugePanelSonderfertigkeitWert();

        debugErzeugePanelSprache();

        debugErzeugePanelSteigern( hauptpanel );

        debugErzeugePanelTalentBonusEntdecker();

        debugErzeugePanelTalentbonusFest();

        debugErzeugePanelTalentbonusFrei();

        debugErzeugePanelVorNachteilWert();
    }

    private PanelAuswahl<VorNachteilWert> debugErzeugePanelVorNachteilWert() {
        
        getPanelVorNachteilWert().setElemente( 
                "ActionFernwaffeneditor.Name", 
                VorNachteilWert.erzeugeWerte( VorNachteil.getVorteile() ), 
                VorNachteilWert.NAME_COMPARATOR );
        
        return getPanelVorNachteilWert();
    }
    
    public void debugZeigePanelVorNachteilWert() {
        
        anzeigen( debugErzeugePanelVorNachteilWert() );
    }

    private PanelTalentBonusFrei debugErzeugePanelTalentbonusFrei() {
        
        getPanelTalentBonusFrei().setBonus( 
                new Held(), 
                new TalentbonusFrei( 7, Talentgruppe.WISSEN ) );
        
        return getPanelTalentBonusFrei();
    }

    public void debugZeigePanelTalentBonusFrei() {
        
        anzeigen( debugErzeugePanelTalentbonusFrei() );
    }

    private PanelTalentBonusFest debugErzeugePanelTalentbonusFest() {
        
        getPanelTalentBonusFest().setBonus(
                new Held(),
                new TalentbonusFest(
                        TalentListe.FREMDSPRACHEN,
                        Arrays.asList( 4, 2, 1 ) ) );
        
        return getPanelTalentBonusFest();
    }

    public void debugZeigePanelTalentBonusFest() {
        
        anzeigen( debugErzeugePanelTalentbonusFest() );
    }

    private PanelAuswahl<Sprache> debugErzeugePanelSprache() {
        
        getPanelSprache().setElemente( 
                "ActionFernwaffeneditor.Name",
                Util.< Talent, Sprache > konvertieren( Talentgruppe.SPRACHEN ),
                Talent.NAME_COMPARATOR );
        
        return getPanelSprache();
    }

    public void debugZeigePanelSprache() {
        
        anzeigen( debugErzeugePanelSprache() );
    }

    private PanelAuswahl<SonderfertigkeitWert> debugErzeugePanelSonderfertigkeitWert() {
        
        getPanelSonderfertigkeitWert().setElemente(
                "ActionFernwaffeneditor.Name",
                SonderfertigkeitWert
                        .erzeugeWerte( Sonderfertigkeitgruppe.KAMPF ),
                SonderfertigkeitWert.NAME_COMPARATOR );
        
        return getPanelSonderfertigkeitWert();
    }

    public void debugZeigePanelSonderfertigkeitWert() {
        
        anzeigen( debugErzeugePanelSonderfertigkeitWert() );
    }

    private PanelAuswahl<Sonderfertigkeit> debugErzeugePanelSonderfertigkeit() {
        
        getPanelSonderfertigkeit().setElemente( "ActionFernwaffeneditor.Name",
                Sonderfertigkeitgruppe.ALLGEMEIN,
                Sonderfertigkeit.NAME_COMPARATOR );
        
        return getPanelSonderfertigkeit();
    }

    public void debugZeigePanelSonderfertigkeit() {
        
        anzeigen( debugErzeugePanelSonderfertigkeit() );
    }

    private PanelTalentBonusFestEntdecker2 debugErzeugePanelTalentBonusEntdecker() {


        Held held = new Held();

        final Datenspeicher speicher = Datenspeicher.getInstanz();

        held.setRasse( speicher.getRasse( "Rasse.Mittellaender" ) );
        held.setKultur( speicher.getKultur( "Kultur.Horasreich" ) );
        held.setProfession( speicher.getProfession( "Profession.Entdecker" ) );

        held.einfuegenProfession( new TalentWert( Talent.FEINMECHANIK, 1 ) );
        held.einfuegenProfession( new TalentWert( Talent.GEOGRAPHIE, 3 ) );
        held.einfuegenVorgegeben( new TalentWert( Talent.GESCHICHTSWISSEN, 1 ) );
        held.einfuegenProfession( new TalentWert( Talent.GESCHICHTSWISSEN, 4 ) );
        held.einfuegenVorgegeben( new TalentWert( Talent.GOETTER_KULTE, 2 ) );
        held.einfuegenProfession( new TalentWert( Talent.GOETTER_KULTE, 4 ) );
        held.einfuegenProfession( new TalentWert( Talent.HERALDIK, 1 ) );
        held.einfuegenProfession( new TalentWert( Talent.KARTOGRAPHIE, 5 ) );
        held.einfuegenProfession( new TalentWert( Talent.MAGIEKUNDE, 3 ) );
        held.einfuegenVorgegeben( new TalentWert( Talent.MECHANIK, 1 ) );
        held.einfuegenProfession( new TalentWert( Talent.MECHANIK, 1 ) );
        held.einfuegenVorgegeben( new TalentWert( Talent.RECHTSKUNDE, 1 ) );
        held.einfuegenProfession( new TalentWert( Talent.RECHTSKUNDE, 1 ) );
        held.einfuegenProfession( new TalentWert( Talent.SPRACHENKUNDE, 3 ) );
        held.einfuegenProfession( new TalentWert( Talent.STERNKUNDE, 2 ) );

        List< Talent > talente = new ArrayList< Talent >();
        talente.add( Talent.ANATOMIE );
        talente.add( Talent.ALCHIMIE );
        talente.add( Talent.BERGBAU );
        talente.add( Talent.FEINMECHANIK );
        talente.add( Talent.GEOGRAPHIE );
        talente.add( Talent.GESCHICHTSWISSEN );
        talente.add( Talent.GESTEINSKUNDE );
        talente.add( Talent.GOETTER_KULTE );
        talente.add( Talent.HERALDIK );
        talente.add( Talent.KARTOGRAPHIE );
        talente.add( Talent.MAGIEKUNDE );
        talente.add( Talent.MECHANIK );
        talente.add( Talent.PFLANZENKUNDE );
        talente.add( Talent.RECHTSKUNDE );
        talente.add( Talent.SPRACHENKUNDE );
        talente.add( Talent.STERNKUNDE );
        talente.add( Talent.TIERKUNDE );
        getPanelTalentBonusEntdecker().setBonus(
                held,
                new TalentbonusFest(
                        new TalentListeFest( talente ),
                        Arrays.asList( 5, 4, 2 ) ) );
        
        return getPanelTalentBonusEntdecker();
    }

    public void debugZeigePanelTalentBonusEntdecker() {
        
        anzeigen( debugErzeugePanelTalentBonusEntdecker() );
    }

    private PanelSteigernErschaffung debugErzeugePanelSteigern(
            final Hauptpanel hauptpanel ) {
        
        final Erschaffung erschaffung = new Erschaffung( hauptpanel );
        
        erschaffung.debugSchnellerschaffung( 
                "Rasse.Mittellaender", 
                "Kultur.MittellaendischeStaedte", 
                "Profession.Ritter" );
        
        getPanelSteigern().setManager( erschaffung.debugGetManager() );
        
        return getPanelSteigern();
    }

    public void debugZeigePanelSteigern( final Hauptpanel hauptpanel ) {
        
        anzeigen( debugErzeugePanelSteigern( hauptpanel ) );
    }


    private PanelRasKulProf debugErzeugePanelRasKulProf() {
        
        getPanelRasKulProf().setHeld( new Held() );
        
        return getPanelRasKulProf();
    }

    public void debugZeigePanelRasKulProf() {
        
        anzeigen( debugErzeugePanelRasKulProf() );
    }

    private PanelLeitzauber debugErzeugePanelLeitzauber() {
        
        final Held held = new Held();
        
        held.einfuegen( new VorNachteilWert( VorNachteil.VOLLZAUBERER ) );
        
        getPanelLeitzauber().setDaten( held, 3 );
        
        return getPanelLeitzauber();
    }

    public void debugZeigePanelLeitzauber() {
        
        anzeigen( debugErzeugePanelLeitzauber() );
    }

    private void debugErzeugePanelGp() {
        
        getPanelGp().setManager( new ErschaffungManager() );
    }

    private PanelBonusAnzeige getPanelBonusAnzeige() {
        if ( null == panelBonusAnzeige ) {
            panelBonusAnzeige = new PanelBonusAnzeige();
            resetter.einfuegen( panelBonusAnzeige );
        }
        return panelBonusAnzeige;
    }
    
    /**
     * @return das Panel auf dem die noch zu verteilenden AP und GP dargestellt
     *         werden.
     */
    public final PanelGp getPanelGp() {
        if ( null == panelGp ) {
            panelGp = new PanelGp();
        }
        return panelGp;
    }

    /**
     * @return das Panel mit dem die zusaetzlichen Leitzauber fuer einen
     *         elfischen Helden gewaehlt werden koennen.
     */
    public final PanelLeitzauber getPanelLeitzauber() {
        if ( null == panelLeitzauber ) {
            panelLeitzauber = new PanelLeitzauber();
        }
        return panelLeitzauber;
    }

    /**
     * @return das Panel mit dem Rasse, Kultur und Profession gewaehlt werden.
     */
    public final PanelRasKulProf getPanelRasKulProf() {
        if ( null == panelRasKulProf ) {
            panelRasKulProf = new PanelRasKulProf();
        }
        return panelRasKulProf;
    }

    /**
     * @return das Panel mit dem man eine Sonderfertigkeit aus einer Liste
     *         waehlen kann.
     */
    public final PanelAuswahl< Sonderfertigkeit > getPanelSonderfertigkeit() {
        if ( null == panelSonderfertigkeit ) {
            panelSonderfertigkeit = new PanelAuswahl< Sonderfertigkeit >();
        }
        return panelSonderfertigkeit;
    }

    /**
     * @return das Panel mit dem man eine Sonderfertigkeit aus einer Liste
     *         waehlen kann.
     */
    public final PanelAuswahl< SonderfertigkeitWert > getPanelSonderfertigkeitWert() {
        if ( null == panelSonderfertigkeitWert ) {
            panelSonderfertigkeitWert = new PanelAuswahl< SonderfertigkeitWert >();
        }
        return panelSonderfertigkeitWert;
    }

    /**
     * @return Ein Panel, das es erlaubt eine Sprache aus einer Liste von
     *         Sprachen zu waehlen. Wird benutzt um die Zweitsprache von Zwergen
     *         zu waehlen.
     */
    public final PanelAuswahl< Sprache > getPanelSprache() {
        if ( null == panelSprache ) {
            panelSprache = new PanelAuswahl< Sprache >();
        }
        return panelSprache;
    }

    /**
     * @return das Panel mit dem im letzten Schritt der Erschaffung die Werte
     *         des Helden gesteigert werden.
     */
    public final PanelSteigernErschaffung getPanelSteigern() {
        if ( null == panelSteigern ) {
            panelSteigern = new PanelSteigernErschaffung();
        }
        return panelSteigern;
    }

    /**
     * @return das Panel mit dem eine Auswahl fuer den speziellen Bonus
     *         getroffen werden kann, welchen die Profession Entdecker vergibt.
     */
    public final PanelTalentBonusFestEntdecker2 getPanelTalentBonusEntdecker() {

        panelTalentBonusEntdecker = new PanelTalentBonusFestEntdecker2();

        return panelTalentBonusEntdecker;
    }

    /**
     * @return das Panel mit dem einen Auswahl fuer einen Bonus wie '4 Punkte
     *         auf Oloarkh, Ologhaijan oder Thorwalsch verteilen' getroffen
     *         werden kann.
     */
    public final PanelTalentBonusFest getPanelTalentBonusFest() {
        if ( null == panelTalentBonusFest ) {
            panelTalentBonusFest = new PanelTalentBonusFest();
        }
        return panelTalentBonusFest;
    }

    /**
     * @return das Panel mit dem eine Auswahl fuer einen Bonus wie 'Singen,
     *         Tanzen oder Zechen +5' getroffen werden kann.
     */
    public final PanelTalentBonusFrei getPanelTalentBonusFrei() {
        if ( null == panelTalentBonusFrei ) {
            panelTalentBonusFrei = new PanelTalentBonusFrei();
        }
        return panelTalentBonusFrei;
    }

    /**
     * @return das Panel mit dem man einen Vor-/Nachteil auswaehlen kann.
     */
    public final PanelAuswahl< VorNachteilWert > getPanelVorNachteilWert() {
        if ( null == panelVorNachteilWert ) {
            panelVorNachteilWert = new PanelAuswahl< VorNachteilWert >();
        }
        return panelVorNachteilWert;
    }

    public final JPanel getPanelKnopfWeiter() {
        if ( null == panelKnopfWeiter ) {
            panelKnopfWeiter = new JPanel( new FormLayout(
                    "7dlu, F:P:G, R:P:G, 7dlu", "7dlu, P, 7dlu" ) );
            final CellConstraints cc = new CellConstraints();
            panelKnopfWeiter.add( hinweis, cc.xy( 2, 2 ) );
            panelKnopfWeiter.add( knopfWeiter, cc.xy( 3, 2 ) );
        }
        return panelKnopfWeiter;
    }

    public boolean kannGewechseltWerden() {
        // Fragen, ob der Benutzer die Erschaffung wirkliche abbrechen will.
        int antwort = Util.zeigeJaNeinDialog( this, 
                I18n.i18n( ABBRECHEN_FRAGE ) );

        return JOptionPane.YES_OPTION == antwort;
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();

        Uebersetzer.neuUebersetzen( 
                panelBonusAnzeige,
                panelGp, 
                panelLeitzauber, 
                panelRasKulProf,
                panelSonderfertigkeitWert, 
                panelSonderfertigkeit, 
                panelSprache,
                panelSteigern, 
                panelTalentBonusFest,
                panelTalentBonusEntdecker,
                panelTalentBonusFrei, 
                panelVorNachteilWert );
    }
    
    /**
     * @param derManager fuer die aktuelle Charaktererschaffung.
     */
    public final void setManager( final ErschaffungManager derManager ) {
        
        manager = derManager;
    }

    /**
     * @param action
     *            mit welcher der Knopf versehen wird der rechts unten im
     *            Fenster zu sehen ist.
     */
    public final void setWeiterAction( final Action action ) {

        knopfWeiter.setAction( action );
    }
}
