/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui;

import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.erschaffung.ErschaffungManager;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelSteigernErschaffung
extends ResetbaresPanel 
implements Uebersetzbar {

    private static final String EIGENSCHAFTEN = "PanelSteigern.Eigenschaften";

    private static final String BASISWERTE = "PanelSteigern.Basiswerte";

    private static final String ZAUBER = "PanelSteigern.Zauber";

    private static final String VOR_NACHTEILE = "PanelSteigern.VorNachteile";

    private static final String SONDERFERTIGKEITEN = "PanelSteigern.Sonderfertigkeiten";

    private static final String TALENTE = "PanelSteigern.Talente";

    private static final String EIGENSCHAFTEN_BASISWERTE = "PanelSteigern.EigenschaftenBasiswerte";

    private static final String PROBLEME = "PanelSteigern.Probleme";

    private static final String HELDENDATEN = "PanelSteigern.Heldendaten";

    /** Zeigt die Basiswerte an. */
    private final PanelBasiswerte basiswerte;
    
    /** Erlaubt es die Eigenschaften zu waehlen. */
    private final PanelEigenschaften eigenschaften;
    
    /** Der Held, der aktuell gesteigert wird. */
    private Held held;

    /** Die allgemeinen Informationen wie Alter und Haarfarbe. */
    private final PanelHeldendaten heldendaten;
    
    /** Zeigt alle Regelkonflikte bei der Erschaffung an. */
    private final PanelProbleme probleme;

    /** Erlaubt es die Sonderfertigkeiten zu waehlen. */
    private final PanelSonderfertigkeiten sonderfertigkeiten;
    
    /**
     * Das Panel zeigt nur diese Tabbedpane an. Sie enthaelt die ganzen Panel
     * die zur Charaktererschaffung benoetigt werden.
     */
    private final JTabbedPane tabbedPane;
    
    /** Erlaubt es die Talente zu waehlen. */
    private final PanelTalente talente;
    
    /** Uebersetzt teile der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Erlaubt es die Vor-/Nachteile zu waehlen. */
    private final PanelVorNachteile vorNachteile;

    /** Erlaubt es die Zauber zu waehlen. */
    private final PanelZauber zauber;

    PanelSteigernErschaffung() {
        
        super( new BorderLayout() );

        uebersetzer = new Uebersetzer();
        
        basiswerte = new PanelBasiswerte();
        
        eigenschaften = new PanelEigenschaften();
        
        final JScrollPane eigenschaftenBasiswerteScrollPane = 
            erzeugeEigenschaftenBasiswertPanel();
        resetter.einfuegen( eigenschaftenBasiswerteScrollPane );
        
        heldendaten = new PanelHeldendaten();
        resetter.einfuegen( heldendaten );

        final SimpleInternalFrame heldendatenFrame = uebersetzer.erzeugeFrame(
                HELDENDATEN,
                heldendaten );
        
        probleme = new PanelProbleme();

        final SimpleInternalFrame problemeFrame = uebersetzer.erzeugeFrame( 
                PROBLEME,
                probleme );
        
        sonderfertigkeiten = new PanelSonderfertigkeiten( false );
        resetter.einfuegen( sonderfertigkeiten );
        
        tabbedPane = new JTabbedPane();
        resetter.einfuegen( tabbedPane );
        
        talente = new PanelTalente( false );
        resetter.einfuegen( talente );
        
        vorNachteile = new PanelVorNachteile();
        resetter.einfuegen( vorNachteile );
        
        zauber = new PanelZauber( false );
        resetter.einfuegen( zauber );
        
        uebersetzer.erzeugeTab(
                tabbedPane,
                HELDENDATEN,
                heldendatenFrame );

        uebersetzer.erzeugeTab(
                tabbedPane,
                EIGENSCHAFTEN_BASISWERTE,
                erzeugeEigenschaftenBasiswertPanel() );

        uebersetzer.erzeugeTab(
                tabbedPane,
                TALENTE,
                talente );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                SONDERFERTIGKEITEN,
                sonderfertigkeiten );

        uebersetzer.erzeugeTab(
                tabbedPane,
                VOR_NACHTEILE,
                vorNachteile );

        uebersetzer.erzeugeTab(
                tabbedPane,
                ZAUBER,
                zauber );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                PROBLEME,
                problemeFrame );

        add( tabbedPane );
    }

    private JScrollPane erzeugeEigenschaftenBasiswertPanel() {

        final SimpleInternalFrame basiswerteFrame = uebersetzer.erzeugeFrame(
                BASISWERTE, 
                basiswerte );
        
        final SimpleInternalFrame eigenschaftenFrame = uebersetzer.erzeugeFrame(
                EIGENSCHAFTEN, 
                eigenschaften );

        final FormLayout layout = new FormLayout(
                "D:G, 4dlu, D:G",   // spalten
                "F:P" ); // zeilen

        final JPanel panel = new JPanel( layout );
        final CellConstraints cc = new CellConstraints();

        panel.add( eigenschaftenFrame, cc.xy( 1, 1 ) );

        panel.add( basiswerteFrame, cc.xy( 3, 1 )  );
        
        return Util.erzeugeScrollPane( panel );
    }
    
    /**
     * @return der Held der aktuell gesteigert wird.
     */
    public final Held getHeld() {
        return held;
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /**
     * Bestimmt den Manager und damit den Held, der gerade erschaffen wird.
     *
     * @param manager fuer die Erschaffung.
     */
    public final void setManager( final ErschaffungManager manager ) {

        held = manager.getHeld();

        heldendaten.setHeld( held );

        eigenschaften.setHeld( held );

        basiswerte.setHeld( held );

        probleme.setManager( manager );

        sonderfertigkeiten.setManager( manager );

        talente.setHeld( held );

        vorNachteile.setManager( manager );

        zauber.setHeld( held );
    }
}
