/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.action;

import java.io.File;

import javax.swing.JFileChooser;

import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.LookAndFeelAktualisierbar;
import de.bernhardjung.dsaprogramm.util.Util;

public abstract class ActionLadenHeldenbaustein 
extends CharGenAction 
implements LookAndFeelAktualisierbar {

    private final String dateierweiterung;
    
    private JFileChooser fileChooser;

    private final String verzeichnis;

    public ActionLadenHeldenbaustein(
            final Hauptpanel dasHauptpanel,
            final String nameId,
            final String mnemonicId,
            final String dateierweiterung,
            final String dasVerzeichnis ) {

        super( dasHauptpanel, nameId, null, mnemonicId );

        this.dateierweiterung = dateierweiterung;
        
        verzeichnis = dasVerzeichnis;
    }

    protected void actionPerformed() {
        
        if ( null == fileChooser ) {
            
            fileChooser = Util.erzeugeFileChooser( dateierweiterung );
        }
        
        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            final String datenquelle = einstellungen.getDatenquelle();

            fileChooser.setCurrentDirectory(
                    new File( datenquelle, verzeichnis ) );
        }

        final int ergebnis = fileChooser.showOpenDialog( hauptpanel );

        if ( JFileChooser.APPROVE_OPTION == ergebnis ) {

            final File datei = fileChooser.getSelectedFile();

            if ( datei.exists() ) {

                try {

                    laden( datei );

                }  catch ( final Exception ex ) {

                    DialogFehlermeldung.zeigeDialog(
                            "Fehler beim Laden von " + datei.getName(), ex );
                }
            }
        }
    }

    public void aktualisiereLookAndFeel() {
        
        fileChooser = null;
    }
    
    abstract void laden( final File datei ) throws Exception;

}
