/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.action;

import java.io.File;
import java.io.IOException;

import javax.swing.FocusManager;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.LookAndFeelAktualisierbar;
import de.bernhardjung.dsaprogramm.util.Util;

abstract class ActionSpeichern 
extends CharGenAction 
implements LookAndFeelAktualisierbar {

    private JFileChooser fileChooser;

    /**
     * @param nameId
     *            die Id, aus der Lokalisierungsdatei, fuer den Namen.
     * @param mnemonicId
     *            die Id, aus der Lokalisierungsdatei, fuer das Mnemonic.
     */
    ActionSpeichern(
            final Hauptpanel dasHauptpanel,
            final String nameId,
            final String mnemonicId ) {

        super( dasHauptpanel, nameId, null, mnemonicId );
    }

    /**
     * @param nameId
     *            die Id, aus der Lokalisierungsdatei, fuer den Namen.
     * @param icon
     *            der Name des Icons.
     * @param mnemonicId
     *            die Id, aus der Lokalisierungsdatei, fuer das Mnemonic.
     * @param tastenId
     *            die Id, aus der Lokalisierungsdatei, fuer das Tastenkuerzel.
     * @param dasHauptfenster
     *            der Anwendung.
     */
    ActionSpeichern(
            final Hauptpanel dasHauptpanel,
            final String nameId,
            final String icon,
            final String mnemonicId,
            final String tastenId ) {

        super( dasHauptpanel, nameId, icon, null, mnemonicId );
    }

    protected void actionPerformed() {

        speichern();
    }

    private boolean erzeugeDatei( final File datei ) {

        boolean dateiErzeugt = false;

        try {

            dateiErzeugt = datei.createNewFile();

        } catch (final IOException e) {
        }

        return dateiErzeugt;
    }

    /**
     * @return die Dateierweiterung wie '.xml' oder '.rtf'.
     */
    abstract String getDateierweiterung();

    /**
     * @return einen moeglichen Dateinamen fuer die zu speichernde Datei.
     */
    abstract String getDateinameVorschlag();

    /**
     * @return das Verzeichnis, das im Filechooser geoeffnet werden soll.
     */
    abstract String getVerzeichnis();

    @Override
    public void neuUebersetzen() {
        
        super.neuUebersetzen();
        
        fileChooser = null;
    }
    
    /**
     * @return {@code true}, wenn das Speichervorgang nicht durch einen
     *          Fehler beendet wurd, d.h wenn der Benutzer den Vorgang regulaer
     *          beendet. Das kann z.B. durch einen Klick auf den 
     *          Abbrechen-Button im Dateiauswahldialog geschehen. In dem Fall
     *          liefert die Methode auch true, es ist aber nicht gespeichert
     *          worden. 
     */
    public final boolean speichern() {
        
        // Damit der Name der evtl. gerade ins Textfeld eingegeben wurde
        // uebernommen wird.
        FocusManager.getCurrentManager().clearGlobalFocusOwner();

        final String dateiname = getDateinameVorschlag();

        final File verzeichnis = new File( getVerzeichnis() );

        final File datei = new File( verzeichnis, dateiname );

        if ( null == fileChooser ) {
            
            fileChooser = Util.erzeugeFileChooser( getDateierweiterung() );
        }
        
        fileChooser.setSelectedFile( datei );

        File gewaehlteDatei = null;
        
        while ( null == gewaehlteDatei ) {
            
            final int speichern = fileChooser.showDialog( 
                    hauptpanel,
                    (String) getValue( NAME ) );

            if ( speichern != JFileChooser.APPROVE_OPTION ) {
             
                return true;
            }

            gewaehlteDatei = fileChooser.getSelectedFile();

            if ( null == gewaehlteDatei ) {
                
                JOptionPane.showMessageDialog( 
                        hauptpanel, 
                        I18n.i18n( "ActionSpeichern.UngueltigeDatei.Meldung" ) );
            }
        }
    
        String testname = gewaehlteDatei.getName();

        testname = testname.toLowerCase();

        if ( ! testname.endsWith( getDateierweiterung() ) ) {
            // Die Datei hat die falsche Endung.
            // Die richtige wird angehaengt.

            String pfad = gewaehlteDatei.getAbsolutePath();

            pfad = pfad + getDateierweiterung();

            gewaehlteDatei = new File( pfad );
        }

        if ( ! gewaehlteDatei.exists() ) {

            final boolean dateiErzeugt = erzeugeDatei( gewaehlteDatei );

            if (!dateiErzeugt) {

                JOptionPane.showMessageDialog( 
                        hauptpanel,
                        I18n.i18n(  "ActionSpeichern.DateiNichtAngelegt.Meldung" ),
                        I18n.i18n( "ActionSpeichern.DateiNichtAngelegt.Titel" ),
                        JOptionPane.ERROR_MESSAGE );

                return false;
            }

        } else {

            final int ersetzen = JOptionPane.showConfirmDialog(
                    hauptpanel,
                    String.format( 
                            I18n.i18n( "ActionSpeichern.DateiExistiert.Meldung"),
                           gewaehlteDatei.getName() ),
                    I18n.i18n( "ActionSpeichern.DateiExistiert.Titel"),
                    JOptionPane.WARNING_MESSAGE );

            if ( ersetzen == JOptionPane.NO_OPTION ) {
                
                return speichern();
            }
        }

        if ( ! gewaehlteDatei.canWrite() ) {

            JOptionPane.showMessageDialog(
                    hauptpanel,
                    I18n.i18n( "ActionSpeichern.KeineSchreibrechte.Meldung" ),
                    I18n.i18n( "ActionSpeichern.KeineSchreibrechte.Titel"),
                    JOptionPane.ERROR_MESSAGE );

            return false;
        }

        // Speichert das Dokument
        speichern( gewaehlteDatei );

        final File gewaehltesVerzeichnis = gewaehlteDatei.getParentFile();

        if ( null != gewaehltesVerzeichnis ) {

            setVerzeichnis( gewaehltesVerzeichnis );
        }
            
        speichernErfolgreich();

        return true;
    }
    
    /**
     * @param datei in die gespeichert werden soll.
     */
    abstract boolean speichern( final File datei );

    /**
     * Wird aufgerufen, wenn der Speichernvorgang erfolgreich war.
     */
    void speichernErfolgreich() {}
    
    /**
     * @param verzeichnis
     *            das im Filechooser geoeffnet werden soll.
     */
    abstract void setVerzeichnis( final File verzeichnis );
    
    public final void aktualisiereLookAndFeel() {
        
        fileChooser = null;
    }
}
