/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.bernhardjung.dsaprogramm.gui.bonus;

import java.util.List;

import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelOffeneBoni extends JPanel implements Uebersetzbar {
    
    /** "Noch zu verteilende Boni." */
    private static final String UEBERSCHRIFT = "PanelOffeneBoni.Ueberschrift";
    
    /**
     * Das Model fuer die Liste die die noch offenen Boni anzeigt.
     */
    private final CharGenListModel< Object, Object > boniModel;
    
    /**
     * Die Liste, die die offenen Boni anzeigt.
     */
    private final JList boniListe;
    
    /**
     * Zeigt den in der Liste gewaehlten Talentbonus an.
     */
    private final JTextArea bonusTextArea;
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    public PanelOffeneBoni() {
    
        uebersetzer = new Uebersetzer();
        
        boniModel = new CharGenListModel< Object, Object >();
        
        boniListe = uebersetzer.erzeugeListe( boniModel );
        boniListe.addListSelectionListener( new ListSelectionListener() {
            public void valueChanged( ListSelectionEvent e ) {
                bonusAnzeigen();
            }
        } );
        
        bonusTextArea = new JTextArea();
        bonusTextArea.setLineWrap( true );
        bonusTextArea.setWrapStyleWord( true );
        
        aufbauenBenutzeroberflaeche();
    }
    
    private void aufbauenBenutzeroberflaeche() {
        
        final FormLayout layout = new FormLayout( 
                "7dlu, F:P:G, 7dlu", 
                "P, 4dlu, F:P:G(0.4), 4dlu, F:P:G(0.6)" );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();
        
        add( uebersetzer.erzeugeUeberschrift( UEBERSCHRIFT ), cc.xy( 2, 1 ) );
        
        add( Util.erzeugeScrollPane( bonusTextArea ), cc.xy( 2, 3 ) );
        
        add( Util.erzeugeScrollPane( boniListe ), cc.xy( 2, 5 ) );
    }
    
    /**
     * Wird aufgerufen, wenn ein Bonus in der Liste angeklickt wird und zeigt
     * den Bonus im Textfeld an.
     */
    private void bonusAnzeigen() {
        
        final Object bonus = boniListe.getSelectedValue();
        
        if ( null != bonus ) {
            
            bonusTextArea.setText( bonus.toString() );
        
        } else {
            
            bonusTextArea.setText( "" );
        }
    }
    
    public final void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
        
        bonusAnzeigen();
    }
    
    /**
     * @param boni die im Panel angezeigt werden sollen.
     */
    public final void setBoni( final List< Object > boni ) {
        
        boniModel.setElements( boni );
        
        bonusAnzeigen();
    }
}
