/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Collections;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.bausteine.Kultur;
import de.bernhardjung.dsaprogramm.bausteine.kosten.KostenFest;
import de.bernhardjung.dsaprogramm.bausteine.util.Geschlecht;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.Voraussetzung;
import de.bernhardjung.dsaprogramm.bausteine.voraussetzungen.VoraussetzungGeschlecht;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaKulturDiverses
extends ResetbaresPanel 
implements Uebersetzbar {
        
    /**
     * Der Text auf der Checkbox, mit der man waehlen kann, dass der Held
     * maennlich oder weiblich sein soll.
     */
    private final JCheckBox geschlechtCheckBox;

    private final JComboBox geschlechtComboBox;

    private final JTextField id;

    private final JSpinner kosten;

    private final JSpinner leitzauber;

    /** Der Text neben der Combobox zum waehlen der Muttersprache. */
    private final JComboBox muttersprache;
    
    private final JTextField name;
    
    /** 
     * Der Text der Checkbox mit der man eine NameFactory fuer die Kultur 
     * waehlen kann 
     */
    private final JCheckBox nameFactoryCheckBox;
    
    private final JComboBox nameFactoryComboBox;

    private final PanelMetaKultur panelKultur;

    
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    public PanelMetaKulturDiverses( final PanelMetaKultur dasPanelKultur ) {

        uebersetzer = new Uebersetzer();
        
        
        
        geschlechtComboBox = uebersetzer.erzeugeComboBox( Geschlecht.values() );
        geschlechtComboBox.setEnabled( false );
        
        geschlechtCheckBox = uebersetzer.erzeugeCheckBox( 
                "PanelMetaKulturDiverses.Voraussetzung" );
        geschlechtCheckBox.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                
                boolean selected = geschlechtCheckBox.isSelected();
                
                geschlechtComboBox.setEnabled( selected );
            }
        } );
        
        id = new JTextField();
        
        final JLabel idLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaKulturDiverses.Id" );

        kosten = Util.erzeugeSpinner( 0, -9, 99, 1 );

        final JLabel kostenLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaKulturDiverses.Kosten" );
        
        leitzauber = Util.erzeugeSpinner( 0, 0, 99, 1 );
        
        final JLabel leitzauberLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaKulturDiverses.Leitzauber" );

        muttersprache = uebersetzer.erzeugeComboBox( 
                Talentgruppe.SPRACHEN,
                Talent.NAME_COMPARATOR );

        final JLabel mutterspracheLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaKulturDiverses.Muttersprache" );
        
        name = new JTextField();

        final JLabel nameLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaKulturDiverses.Name" );
        
        Datenspeicher speicher = Datenspeicher.getInstanz();
        nameFactoryComboBox = uebersetzer.erzeugeComboBox( 
                speicher.getNameFactorys(),
                NameFactory.BEZEICHNUNG_COMPARATOR );
        nameFactoryComboBox.setEnabled( false );
        
        nameFactoryCheckBox = uebersetzer.erzeugeCheckBox( 
                "PanelMetaKulturDiverses.Namen" );
        nameFactoryCheckBox.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                
                boolean selected = nameFactoryCheckBox.isSelected();
                
                nameFactoryComboBox.setEnabled( selected );
            }
        } );
        
        panelKultur = dasPanelKultur;

        final JLabel berechneteKostenLabel = new JLabel();

        final JButton berechnen = uebersetzer.erzeugeButton( 
                "PanelMetaKulturDiverses.Berechnen" );
        berechnen.addActionListener( new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {

                final Kultur kultur = new Kultur( "Kultur" );

                panelKultur.uebertrageDaten( kultur );

                final double kosten = kultur.berechneGpKosten();

                berechneteKostenLabel.setText( 
                        String.format( "%1$.1f", kosten ) );
            }
        });

        final FormLayout layout = new FormLayout( "P, 4dlu, F:P:G" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        builder.append( nameLabel, name );

        builder.append( idLabel, id );

        final FormLayout layout1 = new FormLayout( "P, 4dlu, P, 4dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder1 = new DefaultFormBuilder( layout1 );

        builder1.append( kostenLabel, kosten );
        builder1.append( berechnen, berechneteKostenLabel );

        builder.append( builder1.getPanel(), 3 );

        final FormLayout layout2 = new FormLayout( "P, 4dlu, P" );

        final DefaultFormBuilder builder2 = new DefaultFormBuilder( layout2 );

        builder2.append( mutterspracheLabel, muttersprache );

        builder.append( builder2.getPanel(), 3 );

        final FormLayout layout3 = new FormLayout( "P, 4dlu, P" );

        final DefaultFormBuilder builder3 = new DefaultFormBuilder( layout3 );

        builder3.append( leitzauberLabel, leitzauber );

        builder.append( builder3.getPanel(), 3 );

        final FormLayout layout4 = new FormLayout( "P, 4dlu, P" );

        final DefaultFormBuilder builder4 = new DefaultFormBuilder( layout4 );

        builder4.append( geschlechtCheckBox, geschlechtComboBox );

        builder.append( builder4.getPanel(), 3 );
        
        final FormLayout layout5 = new FormLayout( "P, 4dlu, P" );
        
        final DefaultFormBuilder builder5 = new DefaultFormBuilder( layout5 );
        
        builder5.append( nameFactoryCheckBox, nameFactoryComboBox );

        builder.append( builder5.getPanel(), 3 );
        
        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /**
     * Zeigt einen Kultur im Panel an.
     *
     * @param kultur
     *            die angezeigt werden soll.
     */
    void anzeigen( final Kultur kultur ) {

        kosten.setValue( kultur.getGpKosten( null ) );

        anzeigenId( kultur.getId() );

        leitzauber.setValue( kultur.getAnzahlLeitzauber() );

        muttersprache.setSelectedItem( kultur.getMuttersprache() );

        name.setText( kultur.getName() );

        if ( kultur.hatNameFactory() ) {
            
            nameFactoryCheckBox.setSelected( true );
        
            nameFactoryComboBox.setSelectedItem( kultur.getNameFactory() );
        
        } else {
            
            nameFactoryCheckBox.setSelected( false );
        }
        
        geschlechtCheckBox.setSelected( false );

        for ( final Voraussetzung voraussetzung : kultur.getVoraussetzungen() ) {

            if ( voraussetzung instanceof VoraussetzungGeschlecht ) {

                final Geschlecht geschlecht =
                    ( (VoraussetzungGeschlecht) voraussetzung ).getGeschlecht();

                geschlechtComboBox.setSelectedItem( geschlecht );

                geschlechtCheckBox.setSelected( true );

                break;
            }
        }
    }

    private void anzeigenId( final String idString ) {

        final int index = idString.indexOf( "." );

        final String suffix = idString.substring( 1 + index );

        id.setText( suffix );
    }

    /** @return die fuer die Kultur gewaehlte Id. */
    String getKulturId() {
        return "Kultur." + id.getText().trim();
    }

    /**
     * @return den Namen den der Benutzer fuer die Kultur eingegeben hat.
     */
    String getKulturName() {
        return name.getText();
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Setzt die Eintraege im Panel wieder zurueck.
     */
    public void eingabenLoeschen() {

        geschlechtCheckBox.setSelected( false );

        id.setText( "" );

        kosten.setValue( 0 );

        leitzauber.setValue( 0 );

        name.setText( "" );

    }
    
    /**
     * Uebertraegt die im Panel gemachten Einstellungen auf eine Kultur.
     *
     * @param kultur
     *            die die Einstellungen erhalten soll.
     */
    @SuppressWarnings( "unchecked" )
    void uebertrageDaten( final Kultur kultur ) {

        kultur.setKosten( new KostenFest( (Integer) kosten.getValue() ) );

        kultur.setAnzahlLeitzauber( (Integer) leitzauber.getValue() );

        kultur.setMuttersprache( (Sprache) muttersprache.getSelectedItem() );

        kultur.setName( name.getText() );

        if ( nameFactoryCheckBox.isSelected() ) {
            
            final Object factory = nameFactoryComboBox.getSelectedItem();
            
            kultur.setNameFactory( (NameFactory) factory );
        
        } else {
            
            kultur.setNameFactory( null );
        }
            
        if ( geschlechtCheckBox.isSelected() ) {

            final Geschlecht geschlecht =
                (Geschlecht) geschlechtComboBox.getSelectedItem();

            final Voraussetzung voraussetzung =
                new VoraussetzungGeschlecht( geschlecht );

            kultur.setVoraussetzungen( Arrays.asList( voraussetzung ) );

        } else {

            kultur.setVoraussetzungen( Collections.EMPTY_LIST );
        }
    }
}
