/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.util.BasiswertModifikation;
import de.bernhardjung.dsaprogramm.bausteine.util.EigenschaftsModifikation;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelMetaModifikationen
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Die Liste der Basiswerte fuer die man eine Modifikation angeben kann.
     */
    private final Basiswert[] basiswerte;

    /**
     * Die Checkboxen mit denen gewaehlt werden kann ob es eine Mod. auf einen
     * Basiswert gibt. Die Checkbox an Position i steht fuer den Basiswert der
     * in {@link #basiswerte} an Position i steht.
     */
    private final List< JCheckBox > basiswerteCheckBoxen;
    
    /**
     * Die Spinner mit denen die Hoehe der Modifikation angegeben werden kann.
     * Der Spinner an Position i steht fuer den Basiswert, der in
     * {@link #basiswerte} an Position i steht.
     */
    private final List< JSpinner > basiswerteSpinner;
    
    /**
     * Die Eigenschaften, fuer die man eine Mod. angeben kann.
     */
    private final Eigenschaft[] eigenschaften;
    
    /**
     * Die Checkboxen mit denen gewaehlt werden kann ob es eine Mod. auf eine
     * Eigenschaft gibt. Die Checkbox an Position i steht fuer die Eigenschaft
     * die in {@link #eigenschaften} an Position i steht.
     */
    private final List< JCheckBox > eigenschaftenCheckBoxen;

    /**
     * Die Spinner mit denen die Hoehe der Modifikation angegeben werden kann.
     * Der Spinner an Position i steht fuer die Eigenschaft, die in
     * {@link #eigenschaften} an Position i steht.
     */
    private final List< JSpinner > eigenschaftenSpinner;
    
    
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Die Basiswerte auf die es immer eine Modifikation gibt. */
    private final Collection< Basiswert > vorgegebene;

    PanelMetaModifikationen(
            final Collection< Basiswert > dieVorgegebenen ) {

        uebersetzer = new Uebersetzer();

        
        
        vorgegebene = new HashSet< Basiswert >( dieVorgegebenen );

        eigenschaften = Eigenschaft.getEigenschaften();

        eigenschaftenCheckBoxen = new ArrayList< JCheckBox >();

        eigenschaftenSpinner = new ArrayList< JSpinner >();

        for ( final Eigenschaft eigenschaft : eigenschaften ) {

            eigenschaftenCheckBoxen.add( uebersetzer.erzeugeCheckBox( 
                    eigenschaft.getLokalisierungsschluesselName() ) );

            eigenschaftenSpinner.add( Util.erzeugeSpinner( 0, -99, 99, 1 ) );
        }
        
        basiswerte = new Basiswert[]{ Basiswert.LE,
                                      Basiswert.AU,
                                      Basiswert.MR,
                                      Basiswert.AE,
                                      Basiswert.INI };

        basiswerteCheckBoxen = new ArrayList< JCheckBox >();
        
        basiswerteSpinner = new ArrayList< JSpinner >();
        
        for ( final Basiswert basiswert : basiswerte ) {

            final JCheckBox checkBox = uebersetzer.erzeugeCheckBox( 
                    basiswert.getLokalisierungsschluesselName() );

            final boolean vorgegeben = vorgegebene.contains( basiswert );

            checkBox.setSelected( vorgegeben );

            basiswerteCheckBoxen.add( checkBox );

            basiswerteSpinner.add( Util.erzeugeSpinner( 0, -99, 99, 1 ) );
        }

        aufbauenBenutzeroberflaeche();
    }

    public void anzeigen( final Heldenbaustein baustein ) {

        for ( int i = 0, n = eigenschaften.length ; i < n ; ++i ) {

            if ( baustein.hatModifikation( eigenschaften[ i ] ) ) {

                eigenschaftenCheckBoxen.get( i ).setSelected( true );

                final int mod = baustein.getModifikation( eigenschaften[ i ] );

                eigenschaftenSpinner.get( i ).setValue( mod );

            } else {

                eigenschaftenCheckBoxen.get( i ).setSelected( false );

                eigenschaftenSpinner.get( i ).setValue( 0 );
            }
        }

        for ( int i = 0, n = basiswerte.length ; i < n ; ++i ) {

            if ( baustein.hatModifikation( basiswerte[ i ] ) ) {

                basiswerteCheckBoxen.get( i ).setSelected( true );

                final int mod = baustein.getModifikation( basiswerte[ i ] );

                basiswerteSpinner.get( i ).setValue( mod );

            } else {

                if ( ! vorgegebene.contains( basiswerte[ i ] ) ) {

                    basiswerteCheckBoxen.get( i ).setSelected( false );
                }

                basiswerteSpinner.get( i ).setValue( 0 );
            }
        }
    }

    private void aufbauenBenutzeroberflaeche() {

        final JLabel basiswerteUeberschrift = uebersetzer.erzeugeUeberschrift( 
                "PanelMetaModifikationen.Basiswerte" );

        final JLabel eigenschaftenUeberschrift = 
            uebersetzer.erzeugeUeberschrift( 
                "PanelMetaModifikationen.Eigenschaften" );

        final FormLayout layout = new FormLayout( "P, 4dlu, P, 7dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        builder.append( eigenschaftenUeberschrift, 3 );

        builder.append( basiswerteUeberschrift, 3 );

        for ( int i = 0, n = eigenschaften.length ; i < n ; ++i ) {

            builder.append( eigenschaftenCheckBoxen.get( i ) );

            builder.append( eigenschaftenSpinner.get( i ) );

            if ( i < basiswerte.length ) {

                final JCheckBox checkBox = basiswerteCheckBoxen.get( i );

                if ( checkBox.isSelected() ) {

                    final JLabel label = uebersetzer.erzeugeLabel( 
                            basiswerte[ i ].getLokalisierungsschluesselName() );
                    
                    builder.append( label );

                } else {
                    
                    builder.append( checkBox );
                }

                builder.append( basiswerteSpinner.get( i ) );

            } else {

                builder.nextLine( 1 );
            }
        }
        
        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /** Waehlt alle Modifikationen ab und setzt die Spinner auf 0. */
    public void eingabenLoeschen() {

        for ( int i = 0, n = basiswerte.length; i < n ; ++i ) {

            final boolean gewaehlt = vorgegebene.contains( basiswerte[ i ] );

            basiswerteCheckBoxen.get( i ).setSelected( gewaehlt );

            basiswerteSpinner.get( i ).setValue( 0 );
        }

        for ( int i = 0, n = eigenschaften.length; i < n ; ++i ) {

            eigenschaftenCheckBoxen.get( i ).setSelected( false );

            eigenschaftenSpinner.get( i ).setValue( 0 );
        }
    }
    
    public final void uebertrageDaten( final Heldenbaustein baustein ) {

        final List< BasiswertModifikation > basiswertMod =
            new ArrayList< BasiswertModifikation >();

        for ( int i = 0, n = basiswerte.length ; i < n ; ++i ) {

            final JCheckBox checkBox = basiswerteCheckBoxen.get( i );

            if ( checkBox.isSelected() ) {

                final int mod = (Integer) basiswerteSpinner.get( i ).getValue();

                if ( ( 0 != mod ) || vorgegebene.contains( basiswerte[ i ] ) ) {

                    basiswertMod.add(
                            new BasiswertModifikation( basiswerte[ i ], mod ) );
                }
            }
        }

        baustein.setBasiswertModifikationen( basiswertMod );

        final List< EigenschaftsModifikation > eigenschaftsMod =
            new ArrayList< EigenschaftsModifikation >();

        for ( int i = 0, n = eigenschaften.length ; i < n ; ++i ) {

            if ( eigenschaftenCheckBoxen.get( i ).isSelected() ) {

                final int mod = (Integer) eigenschaftenSpinner.get( i ).getValue();

                if ( 0 != mod ) {

                    eigenschaftsMod.add(
                            new EigenschaftsModifikation(
                                    eigenschaften[ i ], mod ) );
                }
            }
        }

        baustein.setEigenschaftModifikationen( eigenschaftsMod );
    }
}
