/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.namen.NameFactory;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.daten.Datenspeicherer;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.WechselbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaNameFactorys
extends WechselbaresPanel
implements Uebersetzbar {

    /** Das Praefix fuer die ID einer NameFactory. */
    private static final String NAME_FACTORY_ID_PRAEFIX = "Namenformate.";

    /** Das Textfeld, in das man die Bezeichnung der Factory eintragen kann. */
    private final JTextField bezeichnung;
    
    /** Die Liste, die die Factorys anzeigt. */
    private final JList factorys;
    
    /** Das Model fuer die Liste der Factorys. */
    private final CharGenListModel< NameFactory, NameFactory > factoryListModel;

    /** 
     * Wenn der Benutzer die Factorys editiert wird im beim ersten mal ein
     * Hinweis angezeigt, dass er ein Verzeichnis als Datenquelle waehlen muss,
     * damit die Aenderungen gespeichert werden koennen. Nachdem der Hinweis
     * gezeigt wurd wird diese Variable von {@code false} auf {@code true}
     * gesetzt.
     */
    private boolean hinweisGezeigt;
    
    /** Das Textfeld, in das die ID der Factory eingetragen werden muss. */
    private final JTextField id;
    
    /**
     * Der Editor auf dem man die Formate fuer die namen von gemeinen,
     * maennlichen Helden waehlen kann. 
     */
    private final PanelMetaNameFormate maennlich;
    
    /**
     * Der Editor auf dem man die Formate fuer die Namen von adligen,
     * maennlichen Helden waehlen kann.
     */
    private final PanelMetaNameFormate maennlichAdlig;
    
    /**
     * Das Panel in dem man die Namenlisten der Factory bearbeiten kann.
     */
    private final PanelMetaNameFactorysNamenlisten namenlisten;
    
    /**
     * Der Button mit dem die Eingaben in den Editor fuer die Factory 
     * uebernommen werden.
     */
    private final JButton uebernehmen;
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private Uebersetzer uebersetzer;
    
    /**
     * Der Editor auf dem man die Formatvorlagen fuer die Namen von gemeinen
     * weiblichen Helden waehlen kann.
     */
    private final PanelMetaNameFormate weiblich;
    
    /**
     * Der Editor mit dem man die Formate fuer die Namen von adligen weiblichen
     * Helden waehlen kann.
     */
    private final PanelMetaNameFormate weiblichAdlig;
    
    public PanelMetaNameFactorys() {

        uebersetzer = new Uebersetzer();
        
        bezeichnung = new JTextField();
        
        factoryListModel = new CharGenListModel< NameFactory, NameFactory >(
                Datenspeicher.getInstanz().getNameFactorys(),
                NameFactory.BEZEICHNUNG_COMPARATOR );
        
        factorys = uebersetzer.erzeugeListe( factoryListModel );
        factorys.addListSelectionListener( new ListSelectionListener() {
            public void valueChanged( ListSelectionEvent e ) {
                anzeigen();
            }
        } );
        
        hinweisGezeigt = false;

        id = new JTextField();
        id.getDocument().addDocumentListener( new DocumentListener() {
            public void changedUpdate( final DocumentEvent e ) {}

            public void insertUpdate( final DocumentEvent e ) {
                pruefen();
            }
            public void removeUpdate( final DocumentEvent e ) {
                pruefen();
            }
        });
        
        maennlich = new PanelMetaNameFormate();
        
        maennlichAdlig = new PanelMetaNameFormate();

        namenlisten = new PanelMetaNameFactorysNamenlisten();
                
        uebernehmen = uebersetzer.erzeugeButton( 
                "PanelMetaNameFactorys.Uebernehmen" );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                uebernehmen();
            }
        } );
        
        weiblich = new PanelMetaNameFormate();
        
        weiblichAdlig = new PanelMetaNameFormate();
        
        aufbauenBenutzeroberflaeche();
    }

    /**
     * Zeigt die aktuell in der Liste auf der linken Seite gewaehlte Factory
     * im Panel an.
     */
    private void anzeigen() {
        
        final NameFactory factory = getGewaehlteFactory();
        
        if ( null != factory ) {
            
            bezeichnung.setText( factory.getBezeichnung() );
            
            id.setText( factory.getId().replace( NAME_FACTORY_ID_PRAEFIX, "" ));
            
            maennlich.setFormate(  factory.getFormatStringsNamenMaennlich() );
            
            maennlichAdlig.setFormate( 
                    factory.getFormatStringsNamenMaennlichAdlig() );
            
            namenlisten.setNamenlisten( factory.getNamenlisten() );
                
            weiblich.setFormate( factory.getFormatStringsNamenWeiblich() );
            
            weiblichAdlig.setFormate(
                    factory.getFormatStringsNamenWeiblichAdlig() );
        }
            
    }

    private void aufbauenBenutzeroberflaeche() {
        
        final String zeilen = "7dlu, F:P:G, 7dlu";
        
        final String spalten = "7dlu, F:100px:G(0.2), 4dlu, F:P:G(0.8), 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );
        
        setLayout( layout );
        
        final CellConstraints cc = new CellConstraints();

        // Die linke Seite zeigt eine Liste mit den Factories.
        add( erzeugeLinkeSpalte(), cc.xy( 2, 2 ) );
        
        // Auf der rechten Seite befindet sich das Tab in dem man die Daten
        // uber die Factory eingeben kann und wo man die Namensliste eingibt.
        add( erzeugeRechteSpalte(), cc.xy( 4, 2 ) );
    }

    /** 
     * Entfernt die aktuell in der Liste gewaehlte Factory aus der Liste der
     * Factories. 
     */
    private void entfernen() {
        
        final NameFactory factory  = getGewaehlteFactory();
        
        if ( null == factory )
        {
            return;
        }
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();
        
        speicher.entfernen( factory );
        
        speichern();
        
        factoryListModel.removeElement( factory );
    }
    
    /**
     * Erzeugt die Liste der Factories.
     */
    private JComponent erzeugeLinkeSpalte() {
        
        final JButton entfernen = uebersetzer.erzeugeButton( 
                "PanelMetaNameFactorys.Entfernen" );
        entfernen.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                entfernen();
            }
        } );

        final JPanel panel = new JPanel( new BorderLayout() );
        
        final JScrollPane scrollPane = new JScrollPane( factorys );
        
        panel.add( scrollPane, BorderLayout.CENTER );
        
        panel.add( entfernen, BorderLayout.PAGE_END );
        
        return panel;
    }
    
    /**
     * Erzeugt den Inhalt der ersten Karteikarte (Allgemein), auf der 
     * Id und Bezeichnung der Factory eingegeben werden kann, sowie die
     * Formate fuer die Namen.
     */
    private JComponent erzeugePanelAllgemein() {

        final JLabel bezeichnungLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNameFactorys.Bezeichnung" );
        
        final JLabel idLabel = uebersetzer.erzeugeLabel( 
                "PanelMetaNameFactorys.Id" );

        final FormLayout layout = new FormLayout( "P, 4dlu, F:P:G" );
        
        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        builder.setDefaultDialogBorder();
        
        builder.append( idLabel, id );
        
        builder.append( bezeichnungLabel, bezeichnung );
        
        return builder.getPanel();
    }
    
    /**
     * Erzeugt die Karteikarten auf der rechten Seite.
     */
    private JComponent erzeugeRechteSpalte() {

        final JTabbedPane tabbedPaneFormate = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPaneFormate );
        
        uebersetzer.erzeugeTab(
                tabbedPaneFormate, 
                "PanelMetaNameFactorys.Maennlich.Reiter", 
                maennlich );
        
        uebersetzer.erzeugeTab(
                tabbedPaneFormate, 
                "PanelMetaNameFactorys.MaennlichAdlig.Reiter", 
                maennlichAdlig );
        
        uebersetzer.erzeugeTab(
                tabbedPaneFormate, 
                "PanelMetaNameFactorys.Weiblich.Reiter", 
                weiblich );
        
        uebersetzer.erzeugeTab(
                tabbedPaneFormate, 
                "PanelMetaNameFactorys.WeiblichAdlig.Reiter", 
                weiblichAdlig );

        final JTabbedPane tabbedPaneGesamt = new JTabbedPane();
        resetter.einfuegen( tabbedPaneGesamt );
        
        uebersetzer.erzeugeTab(
                tabbedPaneGesamt,
                "PanelMetaNameFactorys.Allgemein.Reiter", 
                erzeugePanelAllgemein() );
        
        uebersetzer.erzeugeTab(
                tabbedPaneGesamt,
                "PanelMetaNameFactorys.Formate.Reiter", 
                tabbedPaneFormate );
        
        uebersetzer.erzeugeTab(
                tabbedPaneGesamt, 
                "PanelMetaNameFactorys.Namenslisten.Reiter", 
                namenlisten );

        final JPanel panel = new JPanel( new BorderLayout() );
        
        panel.add( tabbedPaneGesamt, BorderLayout.CENTER );
        
        panel.add( uebernehmen, BorderLayout.PAGE_END );
        
        return panel;
    }

    /**
     * @return Die Factory, die in der Liste der Factorys gewaehlt ist oder
     *         {@code null}, wenn keine Factory gewaehlt ist.
     */
    private NameFactory getGewaehlteFactory() {
        return (NameFactory) factorys.getSelectedValue();
    }
    
    public boolean kannGewechseltWerden() {
        
        return true;
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /**
     * Prueft die Eingaben im Panel auf Korrektheit.
     *
     * @return {@code true}, wenn alles Ok ist, sonst {@code false}.
     */
    private boolean pruefen() {

        boolean ergebnis = true;

        if ( ! pruefen( id ) ) {
            
            ergebnis = false;
        }
        
        uebernehmen.setEnabled( ergebnis );

        return ergebnis;
    }
    
    /**
     * Prueft ob in das Textfeld etwas eingegeben wurde.
     * 
     * @param textfeld das ueberprueft werden soll.
     * @return {@code true}, wenn in das Textfeld etwas eingegeben wurde, sonst
     *         {@code false}.
     */
    private boolean pruefen( final JTextField textfeld ) {
        
        final boolean ergebnis;
        
        if ( textfeld.getText().trim().equals( "" ) ) {

            textfeld.setBackground( Util.FEHLERFARBE );
            
            ergebnis = false;

        } else {

            textfeld.setBackground( bezeichnung.getBackground() );
            
            ergebnis = true;
        }
        
        return ergebnis;
    }

    /** Loescht alle Eingaben aus dem Editor. */
    private void eingabenLoeschen()
    {
        bezeichnung.setText( "" );
        
        id.setText( "" );
        
        maennlich.reset();
        
        maennlichAdlig.reset();
        
        namenlisten.reset();
        
        weiblich.reset();
        
        weiblichAdlig.reset();
    }
    
    /**
     * Speichert die Factorys auf die Festplatte.
     */
    private void speichern() {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            Datenspeicherer.speichereNameFactorys();

        } else if ( ! hinweisGezeigt ) {

            JOptionPane.showMessageDialog( 
                    this, 
                    I18n.i18n( "PanelMetaNameFactorys.HinweisDatenquelle" ) );
            
            hinweisGezeigt = true;
        }
    }

    private void uebernehmen() {
        
        if ( ! pruefen()  ) {
            
            return;
        }
        
        final String id = NAME_FACTORY_ID_PRAEFIX + this.id.getText().trim();
        
        Datenspeicher speicher = Datenspeicher.getInstanz();
        
        final NameFactory factory = speicher.getNameFactory( id );
        
        factory.setBezeichnung( bezeichnung.getText().trim() );
        
        factory.setFormatStringsNameMaennlich( maennlich.getFormate() );
        
        factory.setFormatStringsNameMaennlichAdlig(
                maennlichAdlig.getFormate() );
        
        factory.setFormatStringsNameWeiblich( weiblich.getFormate() );
        
        factory.setFormatStringsNameWeiblichAdlig(
                weiblichAdlig.getFormate() );
        
        factory.setNamenlisten( namenlisten.getNamenlisten() );
        
        speicher.einfuegen( factory );
        
        speichern();
        
        factoryListModel.setElements( speicher.getNameFactorys() );
        
        eingabenLoeschen();
    }
}
