/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Talentbonus;
import de.bernhardjung.dsaprogramm.bausteine.bonus.TalentbonusFrei;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelMetaTalentbonusFrei 
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Punkte gewaehlt
     * werden, die man auf die Talente verteilen darf.
     */
    private static final String BONUS =
        "PanelMetaTalentbonusFrei.Bonus";

    /**
     * Die Ueberschrift fuer den Abschnitt in dem die Talente gewaehlt
     * werden.
     */
    private static final String TALENTE =
        "PanelMetaTalentbonusFest.Talente";

    /**
     * Der Text auf dem Knopf, mit dem man den im Panel gewaehlten Bonus, in die
     * Liste der Talentboni uebernimmt.
     */
    private static final String UEBERNEHMEN =
        "PanelMetaTalentbonusFrei.Uebernehmen";

    /**
     * Der Text neben dem Spinner in dem man die verteilbaren Talentpunkte
     * einstellt.
     */
    private static final String VERTEILEN =
        "PanelMetaTalentbonusFrei.Verteilen";

    /** Die vom Benutzer, fuer den Bonus, ausgewaehlten Talente. */
    private final JList gewaehlteTalenteListe;

    /** Das ListModel fuer die Liste der gewaehlten Talente. */
    private final CharGenListModel< ElementMitName, Talent > gewaehlteTalenteModel;

    /** Ein Spinner, in dem ein Stufenbonus eingestellt werden kann., */
    private final JSpinner punkte;

    
    
    /** Eine Liste aller Talente. */
    private final JList talente;

    /** Die Liste von Talentboni, in die man, ueber dieses Panel, neue Talentboni einfuegt. */
    private final CharGenListModel< Talentbonus, Talentbonus > talentboni;

    /**
     * Der Knopf mit dem man einen im Panel eingestellten Talentbonus in die
     * Liste der Talentboni uebernimmt.
     */
    private final JButton uebernehmen;

    /** Ubersetzt die angezeigten Texte. */
    private final Uebersetzer uebersetzer;
    
    PanelMetaTalentbonusFrei(
            final CharGenListModel< Talentbonus, Talentbonus > dieTalentboni ) {

        uebersetzer = new Uebersetzer();
        
        
        
        gewaehlteTalenteModel = new CharGenListModel< ElementMitName, Talent >();
        uebersetzer.einfuegen( gewaehlteTalenteModel );

        gewaehlteTalenteListe = uebersetzer.erzeugeListe( 
                gewaehlteTalenteModel );
        gewaehlteTalenteListe.setSelectionMode( 
                ListSelectionModel.SINGLE_SELECTION );
        gewaehlteTalenteListe.addListSelectionListener(
                new ListSelectionListener(){
                    public void valueChanged( final ListSelectionEvent e ) {
                        
                        if ( e.getValueIsAdjusting() ) {
                            
                            return;
                        }
                        
                        final int index = gewaehlteTalenteListe.getSelectedIndex();
                        
                        if ( index >= 0 ) {
                            
                            gewaehlteTalenteModel.removeElementAt( index );
                            
                            pruefen();
                        }
                    }
                });

        punkte = Util.erzeugeSpinner( 1, 1, 99, 1 );

        final Vector< Talent > talentliste = new Vector< Talent >();
        for ( final Talentgruppe gruppe: Talentgruppe.getTalentgruppenAlle() ) {
            talentliste.addAll( gruppe );
        }
        talente = new JList( talentliste );
        talente.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
        talente.addMouseListener( new MouseAdapter() {
            @Override
            public void mouseClicked( final MouseEvent e ) {
                final Object talent = talente.getSelectedValue();
                if ( null != talent ) {
                    if ( ! gewaehlteTalenteModel.contains( talent ) ) {
                        gewaehlteTalenteModel.addElement( talent );
                        pruefen();
                    }
                }
            }
        } );

        talentboni = dieTalentboni;
        
        uebernehmen = uebersetzer.erzeugeButton( UEBERNEHMEN );
        uebernehmen.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {

                final Talentbonus bonus = new TalentbonusFrei(
                        (Integer) punkte.getValue(),
                        gewaehlteTalenteModel.getElements() );

                talentboni.addElement( bonus );

                gewaehlteTalenteModel.clear();

                pruefen();
            }
        });
        uebernehmen.setEnabled( false );

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final JScrollPane talenteScrollPane = 
            Util.erzeugeScrollPane( talente, 150, 200 );
        
        final JScrollPane gewaehlteTalenteScrollPane =
            Util.erzeugeScrollPane( gewaehlteTalenteListe, 150, 100 );
        
        final JPanel spinnerPanel = new JPanel(
                new FormLayout( "P, 4dlu, P", "P" ) );

        final CellConstraints cc = new CellConstraints();

        spinnerPanel.add( 
                uebersetzer.erzeugeLabel( VERTEILEN ), 
                cc.xy( 1, 1 ) );

        spinnerPanel.add( punkte, cc.xy( 3, 1 ) );

        final String spalten = "7dlu, F:P:G, 4dlu, F:P:G, 7dlu";

        final String zeilen =
            "7dlu, P, 4dlu, F:P:G, 4dlu, P, 4dlu, P, 4dlu, P, 7dlu";

        final FormLayout layout = new FormLayout( spalten, zeilen );

        final JPanel inhalt = new JPanel( layout );

        inhalt.add( talenteScrollPane, cc.xywh( 2, 2, 1, 7 ) );

        inhalt.add( uebernehmen, cc.xy( 2, 10 ) );

        inhalt.add( uebersetzer.erzeugeSeparator( TALENTE ), cc.xy( 4, 2 ) );

        inhalt.add( gewaehlteTalenteScrollPane,
                    cc.xy( 4, 4 ) );

        inhalt.add( uebersetzer.erzeugeSeparator( BONUS ), cc.xy( 4, 6 ) );

        inhalt.add( spinnerPanel, cc.xy( 4, 8 ) );
        
        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /** Uebersetzt die Benutzeroberflaeche neu. */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
        
        final Vector< Talent > talentliste = new Vector< Talent >();
        for ( final Talentgruppe gruppe: Talentgruppe.getTalentgruppenAlle() ) {
            talentliste.addAll( gruppe );
        }
        talente.setListData( talentliste );
    }

    private void pruefen() {

        final int anzahlTalente = gewaehlteTalenteModel.getSize();

        uebernehmen.setEnabled( anzahlTalente > 1 );
    }

    /** Loescht die gewaehlten Talente. */
    public void eingabenLoeschen() {
        gewaehlteTalenteModel.clear();
    }
}
