/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JTabbedPane;

import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteVariante;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;

class PanelMetaVerbilligteVarianten 
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Der Text auf dem Reiter fuer das Panel mit den Sonderfertigkeiten
     * Kulturkunde, Ortskenntnis und Ruestungsgewoehnung.
     */
    private static final String VERSCHIEDENE =
        "PanelMetaVerbilligteVarianten.Verschiedene";

    
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;

    private final PanelMetaVerbilligteVariantenListe verschiedene;

    private final PanelMetaVerbilligteVariantenListe talentspezialisierungen;

    private final PanelMetaVerbilligteVariantenListe waffenspezialisierungen;
    
    PanelMetaVerbilligteVarianten() {

        uebersetzer = new Uebersetzer();
    
        
        
        final List< Sonderfertigkeit > verschiedeneSf = Arrays.asList(
                Sonderfertigkeit.KULTURKUNDE,
                Sonderfertigkeit.ORTSKENNTNIS,
                Sonderfertigkeit.RUESTUNGSGEWOEHNUNG_I );

        verschiedene = new PanelMetaVerbilligteVariantenListe( verschiedeneSf );
        resetter.einfuegen( verschiedene );

        talentspezialisierungen = new PanelMetaVerbilligteVariantenListe(
                Sonderfertigkeitgruppe.TALENTSPEZIALISIERUNG );
        resetter.einfuegen( talentspezialisierungen );

        waffenspezialisierungen = new PanelMetaVerbilligteVariantenListe(
                Sonderfertigkeitgruppe.WAFFENSPEZIALISIERUNG );
        resetter.einfuegen( waffenspezialisierungen );

        final JTabbedPane tabbedPane = new JTabbedPane();
        resetter.einfuegen( tabbedPane );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                VERSCHIEDENE,
                verschiedene );
        
        uebersetzer.erzeugeTab( 
                tabbedPane,
                Sonderfertigkeitgruppe.TALENTSPEZIALISIERUNG.getLokalisierungsschluesselKuerzel(),
                talentspezialisierungen );
        
        uebersetzer.erzeugeTab( 
                tabbedPane,
                Sonderfertigkeitgruppe.WAFFENSPEZIALISIERUNG.getLokalisierungsschluesselKuerzel(),
                waffenspezialisierungen );
        
        setLayout( new BorderLayout() );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Heldenbaustein baustein ) {

        verschiedene.anzeigen( baustein.getVerbilligteVarianten() );

        talentspezialisierungen.anzeigen( baustein.getVerbilligteVarianten() );

        waffenspezialisierungen.anzeigen( baustein.getVerbilligteVarianten() );
    }

    public final List< VerbilligteVariante > getVerbilligteVarianten() {

        final List< VerbilligteVariante > verbilligte =
            new ArrayList< VerbilligteVariante >();

        verbilligte.addAll( verschiedene.getVerbilligteVarianten() );

        verbilligte.addAll( talentspezialisierungen.getVerbilligteVarianten() );

        verbilligte.addAll( waffenspezialisierungen.getVerbilligteVarianten() );

        return verbilligte;
    }
    
    /** Uebersetzt das Panel neu. */
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /** Waehlt alle Eintraege im Panel ab. */
    public void eingabenLoeschen() {
        
        verschiedene.eingabenLoeschen();

        talentspezialisierungen.eingabenLoeschen();

        waffenspezialisierungen.eingabenLoeschen();
    }
}
