/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.Document;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteVariante;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelMetaVerbilligteVariantenListe
extends ResetbaresPanel
implements ActionListener, DocumentListener, ItemListener, Uebersetzbar {

    private static final String BESCHREIBUNG =
        "PanelMetaVerbilligteVariantenListe.Beschreibung";

    private static final String HINWEIS =
        "PanelMetaVerbilligteVariantenListe.Hinweis";

    private static final String SONDERFERTIGKEIT =
        "PanelMetaVerbilligteVariantenListe.Sonderfertigkeit";

    private static final String VARIANTE =
        "PanelMetaVerbilligteVariantenListe.Variante";

    private final List< JTextField > beschreibungen;

    private final Map< Document, Integer > documentIndex;

    
    
    private final List< Sonderfertigkeit > sonderfertigkeiten;

    private final List< JCheckBox > sonderfertigkeitenCheckBoxen;

    /** Ubersetzt die angezeigten Texte. */
    private final Uebersetzer uebersetzer;
    
    private final List< JComboBox > varianten;

    PanelMetaVerbilligteVariantenListe(
            final List< Sonderfertigkeit > dieSonderfertigkeiten ) {

        uebersetzer = new Uebersetzer();
        
        
        
        beschreibungen =
            new ArrayList< JTextField >( dieSonderfertigkeiten.size() );

        documentIndex = new HashMap< Document, Integer >();

        sonderfertigkeiten =
            new ArrayList< Sonderfertigkeit >( dieSonderfertigkeiten );

        sonderfertigkeitenCheckBoxen =
            new ArrayList< JCheckBox >( sonderfertigkeiten.size() );

        varianten = new ArrayList< JComboBox >( sonderfertigkeiten.size() );

        for ( int i = 0, n = sonderfertigkeiten.size(); i < n ; ++i) {

            final Sonderfertigkeit sonderfertigkeit =
                sonderfertigkeiten.get( i );

            final JTextField textField = new JTextField();

            final Document document = textField.getDocument();

            document.addDocumentListener( this );

            documentIndex.put( document, i );

            beschreibungen.add( textField );


            final JCheckBox checkBox =
                new JCheckBox( sonderfertigkeit.getName() );

            checkBox.setName( String.valueOf( i ) );

            checkBox.addActionListener( this );

            sonderfertigkeitenCheckBoxen.add( checkBox );


            final JComboBox comboBox = new JComboBox(
                    new Vector< Object >(
                            sonderfertigkeit.getVarianten( null ) ) );

            comboBox.setEditable( sonderfertigkeit.hatFreieVarianten() );

            comboBox.addItemListener( this );

            comboBox.setName( String.valueOf( i ) );

            varianten.add( comboBox );

            update( i );
        }

        final FormLayout layout =
            new FormLayout( "P, 4dlu, F:100px:G, 4dlu, F:100px:G" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        builder.append( uebersetzer.erzeugeUeberschrift( SONDERFERTIGKEIT ) );

        builder.append( uebersetzer.erzeugeUeberschrift( VARIANTE ) );

        builder.append( uebersetzer.erzeugeUeberschrift( BESCHREIBUNG ) );

        for ( int i = 0, n = varianten.size() ; i < n ; ++i ) {

            builder.append( sonderfertigkeitenCheckBoxen.get( i ) );

            if ( sonderfertigkeiten.get( i ).hatFreieVarianten() ) {

                builder.append( varianten.get( i ) );

            } else {

                builder.nextColumn( 2 );
            }

            builder.append( beschreibungen.get( i ) );
        }
        
        final JScrollPane scrollPane =
            Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    /**
     * Wird aufgerufen, wenn eine Checkbox angeklickt wird.
     */
    public void actionPerformed( final ActionEvent e ) {

        final JCheckBox checkBox = (JCheckBox) e.getSource();

        final int index = Integer.valueOf( checkBox.getName() );

        update( index );
    }

    public final void anzeigen( final List< VerbilligteVariante > verbilligte ) {

        for ( final JCheckBox checkBox : sonderfertigkeitenCheckBoxen ) {

            checkBox.setSelected( false );
        }

        for ( final VerbilligteVariante v : verbilligte ) {

            final Sonderfertigkeit sonderfertigkeit =
                v.getWert().getSonderfertigkeit();

            final int index = sonderfertigkeiten.indexOf( sonderfertigkeit );

            if ( index >= 0 ) {

                sonderfertigkeitenCheckBoxen.get( index ).setSelected( true );

                Variante variante = v.getVariante();
                
                if ( variante.hatSchluesselOderName() ) {
                    
                    varianten.get( index ).setSelectedItem( 
                            variante.getSchluesselOderName() );
                
                } else {
                    
                    varianten.get( index ).setSelectedItem( variante );
                }

                String beschreibung = v.getBeschreibungId();
                if ( null == beschreibung ) {
                    beschreibung = "";
                }

                beschreibungen.get( index ).setText( beschreibung );
            }
        }
    }

    public void changedUpdate( final DocumentEvent e ) {
        // ignorieren. Wird eh nicht aufgerufen.
    }

    /**
     * Wird aufgerufen, wenn Text in ein Textfeld eingegeben wird.
     */
    public void insertUpdate( final DocumentEvent e ) {

        final Document document = e.getDocument();

        final int index = documentIndex.get( document );

        update( index );
    }

    /**
     * Wird aufgerufen, wenn sich die Auswahl einer Combobox aendert.
     */
    public void itemStateChanged( final ItemEvent e ) {

        final JComboBox comboBox = (JComboBox) e.getSource();

        final int index = Integer.valueOf( comboBox.getName() );

        update( index );
    }

    /**
     * Wird aufgerufen, wenn ein Zeichen aus einem Textfeld geloescht wird.
     */
    public void removeUpdate( final DocumentEvent e ) {

        final Document document = e.getDocument();

        final int index = documentIndex.get( document );

        update( index );
    }

    public final List< VerbilligteVariante > getVerbilligteVarianten() {

        final List< VerbilligteVariante > liste =
            new ArrayList< VerbilligteVariante >();

        for ( int i = 0, n = sonderfertigkeiten.size(); i < n ; ++i ) {

            Object objekt = varianten.get( i ).getSelectedItem();
            
            final Variante variante;
            
            if ( objekt instanceof String ) {

                String string = ((String) objekt).trim();
                
                if ( string.equals( "" ) ) {
                    
                    variante = null;
                
                } else {
                    
                    variante = new Variante( string );
                }
            
            } else {
                
                variante = (Variante) objekt;
            }

            String beschreibung = beschreibungen.get( i).getText().trim();

            if ( beschreibung.equals( "" ) ) {

                beschreibung = null;
            }

            if ( sonderfertigkeitenCheckBoxen.get( i ).isSelected()
              && ( ( null != variante ) || ( null != beschreibung ) ) ) {

                final Sonderfertigkeit sonderfertigkeit = 
                    sonderfertigkeiten.get( i );
                
                liste.add( new VerbilligteVariante(
                        sonderfertigkeit,
                        variante,
                        beschreibung ) );
            }
        }

        return liste;
    }

    /**
     * Sortiert die Sonderfertigkeiten neu und aktualisiert die angezeigten
     * Texte.
     */
    public void neuUebersetzen() {
        
        final List< VerbilligteVariante > gewaehlte = getVerbilligteVarianten();
        
        Collections.sort( sonderfertigkeiten, Sonderfertigkeit.NAME_COMPARATOR );
        
        for ( int i = 0, n = sonderfertigkeiten.size() ; i < n ; ++i ) {
            
            final Sonderfertigkeit sonderfertigkeit = 
                sonderfertigkeiten.get( i );
            
            final JCheckBox checkBox = sonderfertigkeitenCheckBoxen.get( i );
            
            checkBox.setText( sonderfertigkeit.getName() );
            
            final Object[] variantenArray = 
                sonderfertigkeit.getVarianten( null ).toArray();
            
            final ComboBoxModel variantenModel =
                new DefaultComboBoxModel( variantenArray );
            
            varianten.get( i ).setModel( variantenModel );
        }
        
        anzeigen( gewaehlte );
    }
    
    /** Waehlt alle Eintraege im Panel ab. */
    public void eingabenLoeschen() {

        for ( final JCheckBox checkBox : sonderfertigkeitenCheckBoxen ) {

            checkBox.setSelected( false );
        }
    }
    
    public final void update( final int index ) {

        final Object variante = varianten.get( index ).getSelectedItem();

        String varianteString = "";

        if ( variante instanceof String ) {

            varianteString = variante.toString().trim();
        }

        final String beschreibung = beschreibungen.get( index ).getText().trim();

        if ( sonderfertigkeitenCheckBoxen.get( index ).isSelected()
          && varianteString.equals( "" ) && beschreibung.equals( "" ) ) {

            final JTextField textField = (JTextField)
                varianten.get( index ).getEditor().getEditorComponent();

            textField.setBackground( Util.FEHLERFARBE );
            textField.setToolTipText( I18n.i18n(  HINWEIS ) );

            beschreibungen.get( index ).setBackground( Util.FEHLERFARBE );
            beschreibungen.get( index ).setToolTipText( I18n.i18n( HINWEIS ) );

        } else {

            final JTextField textField = (JTextField)
                varianten.get( index ).getEditor().getEditorComponent();

            textField.setBackground( Color.WHITE );
            textField.setToolTipText( null );

            beschreibungen.get( index ).setBackground( Color.WHITE );
            beschreibungen.get( index ).setToolTipText( null );

        }
    }
}
