/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.meta;

import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Beobachter;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Util;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.*;
import java.util.List;

abstract class PanelMetaVorNachteileAbstrakt
extends ResetbaresPanel
implements Beobachter, Uebersetzbar {

    /** Zeigt die GP-Kosten eines Vor-/Nachteils an.*/
    private static final String GP = "PanelMetaVorNachteileAbstrakt.Gp";

    private final Map< VorNachteil, List< Integer > > abhaengigkeiten;

    /**
     * Wird aufgerufen, wenn auf eine Checkbox geklickt wird.
     */
    private final ActionListener actionAnAbwaehlen;

    /**
     * Wird aufgerufen, wenn eine Variante gewaehlt wird.
     */
    private final ActionListener actionVariante;

    /**
     * Die Liste der Elemente, die der Benutzer ausgewaehlt hat.
     */
    final BeobachtbareListe< VorNachteilWert > gewaehlte;

    /**
     * Das Panel wird auf dem PanelMetaVorNachteileAbstrakt angezeigt und
     * enthaelt den eigentlichen Inhalt des Panels.
     */
    private final JPanel inhalt;
    
    /** Die Label, die die GP-Kosten der Vor-/Nachteile anzeigen. */
    private final List< JLabel > kosten;

    /**
     * Wird aufgerufen, wenn die Stufe eines Wertes geaendert wird.
     */
    private final ChangeListener listenerStufe;

    
    
    /**
     * Die Spinner mit denen sich die Stufen der Vor-/Nachteile einstellen
     * lassen. Der Integer Wert gibt die Zeile an in der der Spinner steht.
     */
    private final Map< Integer, JSpinner > stufen;

    /**
     * Die Comboboxen fuer die Varianten. Der Integer-Wert gibt die Zeile an
     * in der die Box steht.
     */
    private final Map< Integer, JComboBox > varianten;

    /** Die Checkboxen um einen Wert zu waehlen. */
    private final List< JCheckBox > vorNachteile;

    /**
     * Die Liste der Werte die vom Panel angezeigt werden.
     */
    final List< VorNachteilWert > werte;

    /**
     * @param liste Die Liste der Werte, die der Benutzer ausgewaehlt hat.
     */
    PanelMetaVorNachteileAbstrakt(
            final BeobachtbareListe< VorNachteilWert > liste ) {

        
        
        abhaengigkeiten = new HashMap< VorNachteil, List<Integer> >();

        actionAnAbwaehlen = new ActionListener() {

            public void actionPerformed( final ActionEvent e ) {

                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf( checkBox.getName() );

                wertAnAbgewaehlt( index, checkBox.isSelected() );
            }
        };

        actionVariante = new ActionListener() {

            public void actionPerformed( final ActionEvent e ) {

                final JComboBox comboBox = (JComboBox) e.getSource();

                final int index = Integer.valueOf( comboBox.getName() );

                varianteGewaehlt( index, comboBox.getSelectedItem() );
            }

        };

        gewaehlte = liste;
        gewaehlte.anmelden( this );

        inhalt = new JPanel();
        
        kosten = new ArrayList< JLabel >();

        listenerStufe = new ChangeListener() {

            public void stateChanged( final ChangeEvent e ) {

                final JSpinner spinner = (JSpinner) e.getSource();

                final int index = Integer.valueOf( spinner.getName() );

                stufeGeaendert( index, (Integer) spinner.getValue() );
            }

        };

        stufen = new HashMap< Integer, JSpinner >();

        varianten = new HashMap< Integer, JComboBox >();

        vorNachteile = new ArrayList< JCheckBox >();

        werte = new ArrayList< VorNachteilWert >();
        
        final JScrollPane scrollPane = Util.erzeugeScrollPane( inhalt );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    void aktualisieren( final int index ) {

        final VorNachteilWert wert = werte.get( index );

        final JCheckBox checkBox = vorNachteile.get( index );

        checkBox.setSelected( gewaehlte.contains( wert ) );

        if ( varianten.containsKey( index ) ) {

            varianten.get( index ).setSelectedItem( wert.getVariante() );
        }

        if ( stufen.containsKey( index ) ) {

            stufen.get( index ).setValue( wert.getStufe() );
        }

        kosten.get( index ).setText( String.format( 
                I18n.i18n( GP ), 
                wert.getGpKosten() ) );
    }

    void aktualisieren( final VorNachteil vorNachteil ) {

        if ( abhaengigkeiten.containsKey( vorNachteil ) ) {

            for ( final int index : abhaengigkeiten.get( vorNachteil ) ) {

                aktualisieren( index );
            }
        }
    }

    void aufbauenBenutzeroberflaeche() {

        inhalt.removeAll();

        kosten.clear();
        stufen.clear();
        varianten.clear();
        vorNachteile.clear();

        initAbhaengigkeiten();

        // Das neue Layout festlegen.
        final GroupLayout layout = new GroupLayout( inhalt );

        layout.setAutoCreateContainerGaps( true );

        layout.setAutoCreateGaps( true );

        inhalt.setLayout( layout );

        final GroupLayout.ParallelGroup hGroup = layout.createParallelGroup();

        layout.setHorizontalGroup( hGroup );

        final GroupLayout.SequentialGroup vGroup = layout.createSequentialGroup();

        layout.setVerticalGroup( vGroup );

        // Alle 5 Zeilen wird ein Trennstrich eingefuegt, das erleichtert dem
        // Benutzer die Orientierung in den Zeilen der Liste.
        final int extraZeilen = 1 + ( werte.size() / 4 );

        // Jetzt die Liste Zeile fuer Zeile aufbauen.
        for ( int i = 0, n = werte.size() + extraZeilen, trennstriche = 0
                ; i < n
                ; ++i ) {

            final GroupLayout.SequentialGroup zeileH = layout.createSequentialGroup();
            final GroupLayout.ParallelGroup zeileV =
                layout.createParallelGroup( GroupLayout.Alignment.CENTER, false );

            if ( ( i % 5 ) == 0 ) {
                // ein Trennstrich wird eingefuegt.

                final JSeparator separator = new JSeparator();

                zeileH.addComponent( separator );
                zeileV.addComponent( separator );

                hGroup.addGroup( zeileH );
                vGroup.addGroup( zeileV );

                ++trennstriche;

                continue;
            }

            // Das ist der index des Wertes in werte, der in der aktuellen
            // Zeile dargestellt wird.
            final int index = i - trennstriche;

            final String indexString = String.valueOf( index );

            final VorNachteilWert wert = werte.get( index );

            final VorNachteil vorNachteil = wert.getVorNachteil();

            // Spalte mit der Checkbox und evtl mit der Combobox aufbauen.
            if ( vorNachteil.hatVarianten() ) {

                // Hat der Wert eine Variante wird neben der Checkbox
                // eine Combobox benoetigt um die Variante waehlen zu koennen.

                final JCheckBox checkBox = new JCheckBox();

                checkBox.setText( vorNachteil.getPraefix() );

                checkBox.setSelected( gewaehlte.contains( wert ) );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionAnAbwaehlen );

                vorNachteile.add( checkBox );

                zeileH.addComponent( checkBox );
                zeileV.addComponent( checkBox );

                final Vector< Object > variantenVector =
                    new Vector< Object >( vorNachteil.getVarianten( null ) );
                final JComboBox comboBox =
                    new JComboBox( variantenVector );

                comboBox.setEditable( vorNachteil.hatFreieVarianten() );

                comboBox.setSelectedItem( wert.getVariante() );

                comboBox.addActionListener( actionVariante );

                varianten.put( index, comboBox );

                comboBox.setName( indexString );

                zeileH.addComponent( comboBox, 100, 100, Short.MAX_VALUE );
                zeileV.addComponent( comboBox );

            } else {

                final JCheckBox checkBox = new JCheckBox( wert.getName() );

                checkBox.setSelected( gewaehlte.contains( wert ) );

                checkBox.setName( indexString );

                checkBox.addActionListener( actionAnAbwaehlen );

                vorNachteile.add( checkBox );

                zeileH.addComponent( checkBox,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        Short.MAX_VALUE );
                zeileV.addComponent( checkBox );

            }

            if ( vorNachteil.hatStufen( wert ) ) {

                final int stufe = wert.getStufe();

                final int schrittweite = vorNachteil.getSchrittweite( wert );

                final JSpinner spinner =
                    Util.erzeugeSpinner( stufe, schrittweite, 99, schrittweite );

                spinner.addChangeListener( listenerStufe );

                spinner.setName( String.valueOf( index ) );
                stufen.put( index, spinner );

                zeileH.addComponent( spinner,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
                zeileV.addComponent( spinner );
            }

            final JLabel k = Util.erzeugeLabelRechtsbuendig();

            k.setText( String.format( 
                    I18n.i18n( GP ), 
                    wert.getGpKosten() ) );

            kosten.add( k );

            zeileH.addComponent( k,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE,
                        GroupLayout.PREFERRED_SIZE );
            zeileV.addComponent( k );

            hGroup.addGroup( zeileH );
            vGroup.addGroup( zeileV );
        }

        if ( 0 < werte.size() ) {

            layout.linkSize( kosten.toArray( new Component[ kosten.size() ] ) );
        }

    }

    abstract void wertAnAbgewaehlt( final int index, final boolean gewaehlt );

    private void initAbhaengigkeiten() {

        abhaengigkeiten.clear();

        for ( int i = 0, n = werte.size(); i < n ; ++i ) {

            final VorNachteil vorNachteil = werte.get( i ).getVorNachteil();

            if ( ! abhaengigkeiten.containsKey( vorNachteil ) ) {

                abhaengigkeiten.put( vorNachteil, new ArrayList< Integer >( 2 ) );
            }

            abhaengigkeiten.get( vorNachteil ).add( i );
        }
    }

    /**
     * Sortiert die Vorteile neu und aktualisiert die angezeigten Texte. 
     */
    public void neuUebersetzen() {

        Collections.sort( werte, VorNachteilWert.NAME_COMPARATOR );
        
        aufbauenBenutzeroberflaeche();
    }
    
    /**
     * Wird aufgerufen, wenn ueber eine Spinner die Stufe eines Wertes
     * veraendert wird.
     *
     * @param index
     *            des Wertes, dessen Stufe geaendert wurde.
     * @param stufe
     *            die fuer den Wert gewaehlt wurde.
     */
    public abstract void stufeGeaendert( final int index, final int stufe );

    /**
     * Wird aufgerufen, wenn in einer Combobox einen Variante gewaehlt wird.
     *
     * @param index
     *            des Wertes fuer den die Variante gewaehlt wurde.
     * @param variante
     *            die gewaehlt wurde.
     */
    public final void varianteGewaehlt( final int index, final Object variante ) {

        if ( null != variante ) {

            final VorNachteilWert wert = werte.get( index );

            wert.setVariante( variante );
        }

        aktualisieren( index );
    }
}
