/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelAp extends JPanel implements Observer, Uebersetzbar {

    /** Der Text auf dem Knopf mit dem man AP addiert. */
    private static final String ADDIEREN = "PanelAp.Addieren";

    /**
     * Der Text neben dem Spinner mit dem man einstellt wie viele AP man
     * addieren will.
     */
    private static final String AP_ADDIEREN = "PanelAp.ApAddieren";

    /**
     * Der Text neben dem Label, das das AP-Guthaben anzeigt.
     */
    private static final String AP_GUTHABEN = "PanelAp.ApGuthaben";

    /** Das Maximum an AP die mit dem Panel verarbeitet werden koennne. */
    private static final int AP_MAXIMUM = 99999;

    /** Der Text neben dem Spinner mit den eingesetzten AP. */
    private static final String EINGESETZTE_AP =
        "PanelAp.EingesetzteAp";

    /** Der Text neben dem Spinner mit den Gesamt-AP. */
    private static final String GESAMT_AP = "PanelAp.GesamtAp";

    /** Das Label, das das AP-Guthaben anzeigt. */
    private final JLabel apGuthaben;

    /** Zeigt die AP an, die der Held bereits ins Steigern investiert hat. */
    private final JSpinner eingesetzteAp;

    /** Zeigt die gesamten AP an, die dem Held zur Verfuegung stehen. */
    private final JSpinner gesamtAp;

    private final JSpinner apAddieren;

    /** Der Held dessen AP angezeigt und veraendert werden. */
    private Held held;

    /** Uebersetzt die Beschriftung des Panels. */
    private final Uebersetzer uebersetzer;
    
    PanelAp() {
        
        uebersetzer = new Uebersetzer();

        apAddieren = Util.erzeugeSpinner( 100, 1, AP_MAXIMUM, 50 );

        apGuthaben = Util.erzeugeLabelRechtsbuendig();

        gesamtAp = Util.erzeugeSpinner( 0, 0, AP_MAXIMUM, 50 );
        gesamtAp.addChangeListener( new ChangeListener(){
           public void stateChanged( final ChangeEvent e ) {
               setGesamtAp();
           }
        });

        eingesetzteAp = Util.erzeugeSpinner( 0, 0, AP_MAXIMUM, 1 );
        eingesetzteAp.addChangeListener( new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                setEingesetzteAp();
            }
        });

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final JButton bApAddieren = uebersetzer.erzeugeButton( ADDIEREN );
        bApAddieren.addActionListener( new ActionListener() {
            public void actionPerformed( final ActionEvent e ) {
                addiereAp();
            }
        });

        final FormLayout layout = new FormLayout( "P, 4dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( layout, this );

        builder.setDefaultDialogBorder();

        builder.append( uebersetzer.erzeugeLabel( GESAMT_AP ) );
        builder.append( gesamtAp );

        builder.nextLine( 1 );
        builder.append( uebersetzer.erzeugeLabel( EINGESETZTE_AP ) );
        builder.append( eingesetzteAp );

        builder.nextLine( 1 );
        builder.append( uebersetzer.erzeugeLabel( AP_GUTHABEN ) );
        builder.append( apGuthaben );

        builder.nextLine( 1 );
        builder.append( uebersetzer.erzeugeLabel( AP_ADDIEREN ) );
        builder.append( apAddieren );
        builder.append( bApAddieren );
    }

    /**
     * Wird aufgerufen, wenn auf den Knopf zum AP addieren geklickt wird.
     */
    private void addiereAp() {

        final int bonus = (Integer) apAddieren.getValue();

        final int ap = held.getApGesamt();

        if ( ( bonus + ap ) < AP_MAXIMUM ) {
            held.addiereApGesamt( bonus );
        }
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /**
     * Wird aufgerufen wenn die Anzahl der Eingesetzten AP im Spinner geaendert
     * wird.
     */
    private void setEingesetzteAp() {
        final int ap = (Integer) eingesetzteAp.getValue();

        held.setApEingesetzt( ap );
    }

    /**
     * Wird aufgerufen wenn die Anzahl der Gesamt-AP im Spinner geaendert wird.
     */
    private void setGesamtAp() {
        final int ap = (Integer) gesamtAp.getValue();

        held.setApGesamt( ap );
    }

    /**
     * @param derHeld dessen AP mit dem Panel angezeigt und veraendert werden.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        update( null, null );
    }

    /**
     * Aktualisiert die AP.
     *
     * @param o ungenutzt.
     * @param arg ungenutzt.
     */
    public final void update( final Observable o, final Object arg ) {

        apGuthaben.setText( String.valueOf( held.getApGuthaben() ) );

        gesamtAp.setValue( held.getApGesamt() );

        eingesetzteAp.setValue( held.getApEingesetzt() );
    }
}
