/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.FernkampfwaffeWert;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.NahkampfwaffeWert;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

class PanelAusruestung extends ResetbaresPanel implements Uebersetzbar {

    /** Der Titel fuer den Frame mit dem Panel der Fernkampfwaffen. */
    private static final String FERNKAMPF =
        "PanelAusruestung.Fernkampf";

    /** Der Titel fuer die Registerkarte mit den Fernkampfwaffen. */
    private static final String FERNKAMPF_REITER =
        "PanelAusruestung.Fernkampf.Reiter";

    /** Der Titel fuer den Frame mit dem Panel der Nahkampfwaffen. */
    private static final String NAHKAMPF =
        "PanelAusruestung.Nahkampf";

    /** Der Titel fuer die Registerkarte mit den Nahkampfwaffen. */
    private static final String NAHKAMPF_REITER =
        "PanelAusruestung.Nahkampf.Reiter";

    /** Der Titel fuer den Frame mit dem Panel der Ruestungen. */
    private static final String RUESTUNG =
        "PanelAusruestung.Ruestung";

    /** Der Titel fuer die Registerkarte mit den Ruestungen. */
    private static final String RUESTUNG_REITER =
        "PanelAusruestung.Ruestung.Reiter";

    /** Der Titel fuer den Frame mit dem Panel der Schilde. */
    private static final String SCHILDE =
        "PanelAusruestung.Schilde";

    /** Der Titel fuer die Registerkarte mit den Schilden. */
    private static final String SCHILDE_REITER =
        "PanelAusruestung.Schilde.Reiter";

    /**
     * Die Panel mit den Fernkampfwaffen. Jedes Panel erlaubt es eine Waffe
     * zu verwalten. 
     */
    private final List< PanelFernkampfwaffe > fernkampfwaffen;

    /**
     * Die Panel fuer die Nahkampfwaffen. Pro Waffe gibt es ein Panel.
     */
    private final List< PanelNahkampfwaffe > nahkampfwaffen;
    
    /**
     * Das Panel fuer die Ruestungen. Auf dem Panel lassen sich mehrere 
     * Ruestungen waehlen. 
     */
    private final PanelRuestungen ruestungen;

    /**
     * Das Panel fuer die Schilde. Wie bei den Ruestungen gibt es ein Panel fuer
     * alle Schilde.
     */
    private final PanelSchilde schilde;

    /** 
     * Uebersetzt die Texte des Panels 
     */
    private final Uebersetzer uebersetzer;
    
    public PanelAusruestung() {

        uebersetzer = new Uebersetzer();

        fernkampfwaffen = new ArrayList< PanelFernkampfwaffe >( 3 );
        for ( int i = 0 ; i < Held.ANZAHL_FERNKAMPFWAFFEN ; ++i ) {
            
            final PanelFernkampfwaffe panel = new PanelFernkampfwaffe();
            
            uebersetzer.einfuegen( panel );
            
            fernkampfwaffen.add( panel );
        }

        nahkampfwaffen = new ArrayList< PanelNahkampfwaffe >( 5 );
        for ( int i = 0 ; i < Held.ANZAHL_NAHKAMPFWAFFEN ; ++i ) {
            
            final PanelNahkampfwaffe panel = new PanelNahkampfwaffe();
            
            uebersetzer.einfuegen( panel );
            
            nahkampfwaffen.add( panel );
        }

        ruestungen = new PanelRuestungen();
        resetter.einfuegen( ruestungen );
        
        schilde = new PanelSchilde();
        resetter.einfuegen( schilde );
        
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( 
                tabbedPane,
                NAHKAMPF_REITER,
                uebersetzer.erzeugeFrame(
                        NAHKAMPF,
                        erzeugePanel( nahkampfwaffen ) ) );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                FERNKAMPF_REITER,
                uebersetzer.erzeugeFrame(
                        FERNKAMPF,
                        erzeugePanel( fernkampfwaffen ) ) );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                RUESTUNG_REITER,
                uebersetzer.erzeugeFrame( RUESTUNG, ruestungen ) );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                SCHILDE_REITER,
                uebersetzer.erzeugeFrame( SCHILDE, schilde ) );

        setLayout( new BorderLayout() );

        add( tabbedPane, BorderLayout.CENTER );
    }

    /**
     * Erzeugt ein Panel, das mehrere Panel vertikal anordnet.
     *
     * @param panels
     *            die untereinander angeordnet werden sollen.
     * @return ein neu erzeugtes Panel.
     */
    private JScrollPane erzeugePanel(
            final List< ? extends JPanel > panels ) {

        final FormLayout layout = new FormLayout( "P:G" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        for ( final JPanel panel : panels ) {

            builder.append( panel );
        }

        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builder.getPanel() );
        
        resetter.einfuegen( scrollPane );
        
        return scrollPane;
    }
    
    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }

    /**
     * Bestimmt den Held der ueber das Panel ausgeruestet wird.
     *
     * @param held
     *            der ausgeruestet werden soll.
     */
    public final void setHeld( final Held held ) {

        final List< FernkampfwaffeWert > fern = held.getFernkampfwaffen();

        for ( int i = 0, n = fernkampfwaffen.size(); i < n ; ++i ) {

            FernkampfwaffeWert fernkampfwaffe = null;

            if ( fern.size() > i ) {

                fernkampfwaffe = fern.get( i );
            }

            fernkampfwaffen.get( i ).setHeld( held, fernkampfwaffe );
        }

        final List< NahkampfwaffeWert > nah = held.getNahkampfwaffen();

        for ( int i = 0, n = nahkampfwaffen.size() ; i < n ; ++i ) {

            NahkampfwaffeWert nahkampfwaffe = null;

            if ( nah.size() > i ) {

                nahkampfwaffe = nah.get( i );
            }

            nahkampfwaffen.get( i ).setHeld( held, nahkampfwaffe );
        }

        ruestungen.setHeld( held );

        schilde.setHeld( held );
    }
}
