/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;
import de.bernhardjung.dsaprogramm.basiswerte.BasiswertWert;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

public class PanelBasiswerteSteigern 
extends ResetbaresPanel 
implements Observer, Uebersetzbar {

    /** Die Ueberschrift fuer die Spalte mit den AP-Kosten. */
    private static final String AP =
        "PanelBasiswerteSteigern.Ap";

    /** Die Ueberschrift fuer die Spalte mit den Basiswerten. */
    private static final String BASISWERT =
        "PanelBasiswerteSteigern.Basiswert";

    /** Die Ueberschrift fuer die Spalte mit der Gesamtstufe der Basiswerte. */
    private static final String GESAMT =
        "PanelBasiswerteSteigern.Gesamtstufe";

    /** Der Text neben der Combobox fuer die Lernmethode. */
    private static final String LERNMETHODE =
            "PanelEigenschaftenSteigern.Lernmethode";

    /**
     * Die Ueberschrift fuer die Spalte mit den Checkboxen fuer die Spezielle
     * Erfahrung.
     */
    private static final String SPEZIELLE_ERFAHRUNG =
        "PanelBasiswerteSteigern.SpezielleErfahrung";

    /** Die Ueberschrift fuer die Spalte mit den zugekauften Punkten. */
    private static final String ZUKAUF =
        "PanelBasiswerteSteigern.Zukauf";

    /** Die Action die zum Steigern eines Basiswerts verwendet wird. */
    private final ChangeListener actionZukauf;

    /** Die Label die die Steigerungskosten anzeigen. */
    private final List< JLabel > ap;

    /** Die Basiswerte die angezeigt werden. */
    private final Basiswert[] basiswerte;

    /** Die Label die die Namen der Basiswerte anzeigen. */
    private final List< JLabel > basiswerteLabel;

    /** Die Label die die Gesamtstufe der Basiswerte anzeigen. */
    private final List< JLabel > gesamt;

    /** Der Knopf mit dem einen grosse Meditation durchgefuehrt werden kann. */
    private final JButton grosseMeditation;

    /** Der Held dessen Werte angezeigt und manipuliert werden. */
    private Held held;

    /** Die Combobox mit der die Lernmethode ausgewaehlt werden kann. */
    private final JComboBox lernmethode;

    /**
     * Die Checkbox mit der einen Spezielle Erfahrung fuer die Ausdauer
     * angegeben werden kann.
     */
    private final JCheckBox spezielleErfahrungAu;

    /**
     * Die Checkbox mit der einen Spezielle Erfahrung fuer die Lebenenergie
     * angegeben werden kann.
     */
    private final JCheckBox spezielleErfahrungLe;

    /** Zeigt die Stufe des Basiswerts an, ohne die zugekauften Punkte. */
    private final List< JLabel > stufenOhneZukauf;

    /** Uebersetzt die Texte im Panel.*/
    private final Uebersetzer uebersetzer;
    
    /** Zeigen die zum Basiswert zugekauften Punkte an. */
    private final List< JSpinner > zukauf;

    public PanelBasiswerteSteigern() {

        uebersetzer = new Uebersetzer();

        ActionListener actionLernmethode = new ActionListener(){
            public void actionPerformed( final ActionEvent e ) {
                update( null, null );
            }
        };

        /*
      Die Action die zum setzen/entfernen einer Speziellen Erfahrung verwendet
      wird.
     */
        Action actionSpezielleErfahrung = new AbstractAction() {
            public void actionPerformed(final ActionEvent e) {

                final JCheckBox checkBox = (JCheckBox) e.getSource();

                final int index = Integer.valueOf(checkBox.getName());

                held.getWert(basiswerte[index]).setSpezielleErfahrung(
                        checkBox.isSelected());
            }
        };

        actionZukauf = new ChangeListener() {
            public void stateChanged( final ChangeEvent e ) {
                final JSpinner spinner= (JSpinner) e.getSource();

                final int index = Integer.valueOf( spinner.getName() );

                final int punkte = (Integer) spinner.getValue();

                final BasiswertWert wert = held.getWert( basiswerte[ index ] );

                wert.setZukauf( punkte, getLernmethode() );
            }
        };

        basiswerte = Basiswert.getBasiswerte();

        ap = new ArrayList< JLabel >( basiswerte.length );

        basiswerteLabel = new ArrayList< JLabel >( basiswerte.length );

        gesamt = new ArrayList< JLabel >( basiswerte.length );

        lernmethode = uebersetzer.erzeugeComboBox(
                Lernmethode.KEINE,
                Lernmethode.FREI );
        resetter.einfuegen( lernmethode );
        lernmethode.addActionListener(actionLernmethode);

        final ActionGrosseMeditation actionGrosseMeditation = 
            new ActionGrosseMeditation();
        uebersetzer.einfuegen( actionGrosseMeditation );
        grosseMeditation = new JButton( actionGrosseMeditation );
        spezielleErfahrungAu = uebersetzer.erzeugeCheckBox(
                null,
                Lernmethode.SPEZIELLE_ERFAHRUNG.getL10nschluesselName(),
                null );
        spezielleErfahrungAu.setAction(actionSpezielleErfahrung);
        spezielleErfahrungAu.setName(
                String.valueOf( Basiswert.AU.getIndex() ) );
        
        spezielleErfahrungLe = uebersetzer.erzeugeCheckBox(
                null,
                Lernmethode.SPEZIELLE_ERFAHRUNG.getL10nschluesselName(),
                null );
        spezielleErfahrungLe.setAction(actionSpezielleErfahrung);
        spezielleErfahrungLe.setName(
                String.valueOf( Basiswert.LE.getIndex() ) );
        
        stufenOhneZukauf = new ArrayList< JLabel >( basiswerte.length );

        zukauf = new ArrayList< JSpinner >( basiswerte.length );

        aufbauenBenutzeroberflaeche();
    }

    /**
     * Erzeugt die Elemente der Benutzeroberflaeche und ordnet sie auf dem Panel
     * an.
     */
    private void aufbauenBenutzeroberflaeche() {

        // Die einzelnen Element erzeugen. Es werden ein paar Elemente zu viel
        // erzeugt.
        for ( int i = 0, n = basiswerte.length ; i < n ; ++i ) {

            ap.add( Util.erzeugeLabelRechtsbuendig() );

            basiswerteLabel.add(  uebersetzer.erzeugeLabel( 
                    basiswerte[ i ].getLokalisierungsschluesselName() ) );

            gesamt.add( Util.erzeugeLabelRechtsbuendig() );

            stufenOhneZukauf.add( Util.erzeugeLabelRechtsbuendig() );

            final JSpinner spinner = Util.erzeugeSpinner( 0, 0, 99, 1 );
            spinner.setName( String.valueOf( i ) );
            spinner.addChangeListener( actionZukauf );
            zukauf.add( spinner );
        }

        final FormLayout layout = new FormLayout(
          //Basiswert   stufe      +      Zukauf    AP        =      Gesamt    SE
                "P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );
        builder.setDefaultDialogBorder();
        builder.setRowGroupingEnabled( true );

        // erste Zeile
        final DefaultFormBuilder builder2 = new DefaultFormBuilder(
                new FormLayout( "P, 2dlu, 200px" ) );
        builder2.append( uebersetzer.erzeugeLabel( LERNMETHODE ) );
        builder2.append( lernmethode );
        builder.append( builder2.getPanel(), 15 );

        builder.append( uebersetzer.erzeugeUeberschrift( BASISWERT ) );
        builder.append( "" );
        builder.append( "" );
        builder.append( uebersetzer.erzeugeUeberschrift( ZUKAUF ) );
        builder.append( uebersetzer.erzeugeUeberschrift( AP ) );
        builder.append( "" );
        builder.append( uebersetzer.erzeugeUeberschrift( GESAMT ) );
        builder.append( uebersetzer.erzeugeUeberschrift( SPEZIELLE_ERFAHRUNG ) );

        for ( int i = 0, n = basiswerte.length; i < n ; ++i ) {

            builder.append( basiswerteLabel.get( i ) );

            builder.append( stufenOhneZukauf.get( i ) );

            if ( basiswerte[ i ].istSteigerbar() ) {

                builder.append( "+" );

                builder.append( zukauf.get( i ) );

                builder.append( ap.get( i ) );

                builder.append( "=" );

                builder.append( gesamt.get( i ) );

                if ( basiswerte[ i ].equals( Basiswert.AU ) ) {

                    builder.append( spezielleErfahrungAu );

                } else if ( basiswerte[ i ].equals( Basiswert.LE ) ) {

                    builder.append( spezielleErfahrungLe );

                } else if ( basiswerte[ i ].equals( Basiswert.AE ) ) {

                    builder.append( grosseMeditation );

                } else {

                    builder.nextLine( 1 );

                }

            } else {

                builder.nextLine( 1 );
            }
        }
        
        final JScrollPane scrollPane;
        
        scrollPane = Util.erzeugeScrollPane( builder.getPanel() );
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );

        add( scrollPane, BorderLayout.CENTER );
    }

    /**
     * @return die aktuell gewaehlte Lernmethode.
     */
    private Lernmethode getLernmethode() {
        return (Lernmethode) lernmethode.getSelectedItem();
    }

    public void neuUebersetzen() {

        uebersetzer.neuUebersetzen();
    }
    
    /**
     * @param derHeld dessen Werte angezeigt und manipuliert werden.
     */
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );

        update( null, null );
    }

    /**
     * Aktualisiert eine Zeile in der Benutzeroberflaeche.
     *
     * @param index des Basiswertes, dessen Zeile aktualisiert werden soll.
     */
    private void update( final int index ) {

        final Basiswert basiswert = basiswerte[ index ];

        final BasiswertWert wert = held.getWert( basiswert );

        final KonfliktBeschreibung beschreibung = wert.pruefen();

        if ( beschreibung.enthaeltKonflikt() ) {

            basiswerteLabel.get( index ).setForeground( Util.FEHLERFARBE );
            basiswerteLabel.get( index ).setToolTipText(
                    beschreibung.toHtml() );

        } else {

            basiswerteLabel.get( index ).setForeground( getForeground() );
            basiswerteLabel.get( index ).setToolTipText( null );

        }

        final int stufe = wert.getStufe();

        final int z = wert.getZukauf();

        stufenOhneZukauf.get( index ).setText( String.valueOf( stufe - z ) );

        if ( basiswert.istSteigerbar() ) {

            zukauf.get( index ).setValue( z );

            ap.get( index ).setText( "(" + wert.getSteigerungskosten( getLernmethode() ) + ")" );

            gesamt.get( index ).setText( String.valueOf( stufe ) );

            if ( basiswert.equals( Basiswert.AE ) ) {

                zukauf.get( index ).setEnabled( held.istZauberer() );

                grosseMeditation.setEnabled(
                        held.hat( Sonderfertigkeit.GROSSE_MEDITATION ) );

            } else if ( basiswert.equals( Basiswert.LE ) ) {

                spezielleErfahrungLe.setSelected( wert.hatSpezielleErfahrung());

            } else if ( basiswert.equals( Basiswert.AU ) ) {

                spezielleErfahrungAu.setSelected( wert.hatSpezielleErfahrung());
            }

        }
    }

    /**
     * Wird aufgerufen, wenn eine Aenderung am Held durchgefuehrt wird.
     *
     * @param o ungenutzt
     * @param arg ungenutzt
     */
    public final void update( final Observable o, final Object arg ) {

        for ( int i = 0, n = basiswerte.length ; i < n ; ++i ) {

            update( i );
        }
    }

    private final class ActionGrosseMeditation 
    extends AbstractAction 
    implements Uebersetzbar {

        private final int AP_KOSTEN_GROSSE_MEDITATION = 400;

        private final String GROSSE_MEDITATION =
            "PanelBasiswerteSteigern.GrosseMeditation";

        ActionGrosseMeditation() {
            super( Sonderfertigkeit.GROSSE_MEDITATION.getName() );
        }

        public void actionPerformed( final ActionEvent e ) {

            final String bonusString = JOptionPane.showInputDialog(
                    PanelBasiswerteSteigern.this,
                    I18n.i18n( GROSSE_MEDITATION ) );

            try {

                final int bonus = Integer.valueOf( bonusString );

                held.getWert( Basiswert.AE ).addiereBonuspunkte( bonus );

                held.addiereApEingesetzt( AP_KOSTEN_GROSSE_MEDITATION );

            } catch ( final NumberFormatException ex ) {
                // ignorieren
            }
        }
        
        public void neuUebersetzen() {
            
            putValue( NAME, Sonderfertigkeit.GROSSE_MEDITATION.getName() );
        }
    }
}
