/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import static de.bernhardjung.dsaprogramm.basis.Held.ANZAHL_SCHILDE;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.ElementMitName;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Schild;
import de.bernhardjung.dsaprogramm.basis.SchildTyp;
import de.bernhardjung.dsaprogramm.basis.SchildWert;
import de.bernhardjung.dsaprogramm.daten.Datenspeicher;
import de.bernhardjung.dsaprogramm.gui.CharGenListModel;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.util.Util;

public class PanelSchilde 
extends ResetbaresPanel
implements Observer, Uebersetzbar {

    private static final String BF = "PanelSchilde.Bf";

    private static final String INI = "PanelSchilde.Ini";

    private static final String NAME = "PanelSchilde.Name";

    private static final String PA = "PanelSchilde.Pa";

    private static final String SCHILD = "PanelSchilde.Schild";

    private static final String TYP = "PanelSchilde.Typ";

    private static final String WM = "PanelSchilde.Wm";

    /** Die Spinner fuer den Bruchfaktor. */
    private final List< JSpinner > bf;

    /** Die Checkboxen, um einen Schild fuer den Helden zu waehlen. */
    private final List< JCheckBox > gewaehlt;

    /** Der Held, der ueber das Panel manipuliert wird. */
    private Held held;

    /** Die Spinner fuer die Initiative. */
    private final List< JSpinner > ini;

    /** Die Textfelder fuer den Namen. */
    private final List< JTextField > name;

    /** Die Paradewerte. */
    private final List< JLabel > pa;

    
    
    /** Die Schilde die ueber das Panel manipuliert. */
    private final List< SchildWert > schilde;

    /** Die Comboboxen mit denen man den Schildtyp waehlt. */
    private final List< JComboBox > schild;

    /**
     * Die Models fuer die Comboboxen in {@link #schild}.  
     */
    private final 
    List< CharGenListModel< ElementMitName, Schild > > schildListModels;

    /** Die ActionListener fuer die Comboboxen in {@link #schild} */
    private final List< ActionListener > schildListener;

    /** 
     * Zum Waehlen des Deckungstyps fuer Fernkampfangriffe. klein, groß, sehr
     * groß. 
     */
    private final List< JComboBox > typ;
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;

    /** Die Textfelder fuer den Waffenmodifikator. */
    private final List< JTextField > wm;

    /** Das Pattern zum ueberpruefen der Eingaben in die Textfelder. */
    private final Pattern wmPattern;

    public PanelSchilde() {

        uebersetzer = new Uebersetzer();

        
        
        final Datenspeicher speicher = Datenspeicher.getInstanz();

        speicher.addObserver( this );
        
        final ChangeListener bfListener = new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                final JSpinner spinner = (JSpinner) e.getSource();

                final int index = Integer.valueOf( spinner.getName() );

                schilde.get( index ).setBf(
                        (Integer) bf.get( index ).getValue() );
            }
        };

        bf = new ArrayList< JSpinner >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final JSpinner spinner = Util.erzeugeSpinner( 0, -9, 99, 1 );

            spinner.setName( String.valueOf( i ) );

            spinner.addChangeListener( bfListener );

            bf.add( spinner );
        }

        gewaehlt = new ArrayList< JCheckBox >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final JCheckBox checkBox = new JCheckBox();

            final int index = i;

            checkBox.addActionListener( new ActionListener() {
                public void actionPerformed( final ActionEvent e ) {
                    final JCheckBox checkBox = (JCheckBox) e.getSource();

                    if ( checkBox.isSelected() ) {

                        held.einfuegen( schilde.get( index ) );

                    } else  {

                        held.entfernen( schilde.get( index ) );
                    }
                }
            });

            gewaehlt.add( checkBox );
        }

        final ChangeListener iniListener = new ChangeListener() {
            public void stateChanged( ChangeEvent e ) {
                final JSpinner spinner = (JSpinner) e.getSource();

                final int index = Integer.valueOf( spinner.getName() );

                schilde.get( index ).setIni(
                        (Integer) ini.get( index ).getValue() );
            }
        };

        ini = new ArrayList< JSpinner >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final JSpinner spinner = Util.erzeugeSpinner( 0, -9, 99, 1 );

            spinner.setName( String.valueOf( i ) );

            spinner.addChangeListener( iniListener );

            ini.add( spinner );
        }

        name = new ArrayList< JTextField >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final JTextField textField = new JTextField();

            final int index = i;

            textField.getDocument().addDocumentListener( new DocumentListener(){

                public void changedUpdate( final DocumentEvent e ) {}

                public void insertUpdate( final DocumentEvent e ) {
                    setName();
                }

                public void removeUpdate( final DocumentEvent e ) {
                    setName();
                }
                private void setName() {
                    schilde.get( index ).setName( name.get( index ).getText() );
                }

            });

            name.add( textField );
        }

        pa = new ArrayList< JLabel >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            pa.add( new JLabel() );
        }

        final List< Schild > schildeListe = speicher.getSchilde();

        schilde = new ArrayList< SchildWert >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final SchildWert wert = new SchildWert();
            
            if ( 0 < schildeListe.size() ) {
                
                wert.setSchild( schildeListe.get( 0 ) );
            }
            
            schilde.add( wert );
        }
        
        schild = new ArrayList< JComboBox >( ANZAHL_SCHILDE );
        schildListModels = 
            new ArrayList< CharGenListModel<ElementMitName,Schild> >(
                    ANZAHL_SCHILDE );
        schildListener = new ArrayList< ActionListener >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final CharGenListModel< ElementMitName, Schild > model =
                new CharGenListModel< ElementMitName, Schild >(
                        schildeListe,
                        Schild.NAME_COMPARATOR );
            
            final JComboBox comboBox = uebersetzer.erzeugeComboBox( model );

            final int index = i;

            final ActionListener listener = new ActionListener() {
                public void actionPerformed( final ActionEvent e ) {

                    final Schild s =
                        (Schild) schild.get( index ).getSelectedItem();

                    schilde.get( index ).setSchild( s );

                    update( index );
                }
            };

            comboBox.addActionListener( listener );

            schild.add( comboBox );
            schildListener.add( listener );
        }

        typ = new ArrayList< JComboBox >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final JComboBox comboBox = uebersetzer.erzeugeComboBox( 
                    SchildTyp.getTypen() );

            final int index = i;

            comboBox.addActionListener( new ActionListener() {
                public void actionPerformed( final ActionEvent e ) {

                    final SchildTyp t =
                        (SchildTyp) typ.get( index ).getSelectedItem();

                    schilde.get( index ).setTyp( t );
                }
            } );

            typ.add( comboBox );
        }

        wm = new ArrayList< JTextField >( ANZAHL_SCHILDE );
        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            final JTextField textField = new JTextField();

            final int index = i;

            textField.getDocument().addDocumentListener( new DocumentListener(){

                public void changedUpdate( final DocumentEvent e ) {}

                public void insertUpdate( final DocumentEvent e ) {
                    setName();
                }

                public void removeUpdate( final DocumentEvent e ) {
                    setName();
                }
                private void setName() {

                    final String wmString = wm.get( index ).getText();

                    final Matcher matcher = wmPattern.matcher( wmString );

                    final JTextField textField = wm.get( index );

                    if ( matcher.matches() ) {

                        textField.setBackground(
                                name.get( index ).getBackground() );

                        final String[] wmArray = wmString.split( "/" );

                        final SchildWert schild = schilde.get( index );

                        schild.setWmAt( toInt( wmArray[ 0 ] ) );

                        schild.setWmPa( toInt( wmArray[ 1 ] ) );

                    } else {

                        textField.setBackground( Util.FEHLERFARBE );
                    }
                }
                private int toInt( String s ) {

                    if ( s.charAt( 0 ) == '+' ) {

                        s = s.substring( 1 );
                    }

                    return Integer.valueOf( s );
                }
            });

            wm.add( textField );
        }

        wmPattern = Pattern.compile( "[+-]?\\d+/[+-]?\\d+" );

        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            update( i );
        }

        aufbauenBenutzeroberflaeche();
    }

    private void aufbauenBenutzeroberflaeche() {

        final FormLayout layout = new FormLayout(
                "P, 4dlu, 200px:G, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, P, 4dlu, max(P;50px), " +
                "4dlu, P" );

        final DefaultFormBuilder builder = new DefaultFormBuilder( layout );

        builder.setDefaultDialogBorder();

        builder.append( "" );
        builder.append( uebersetzer.erzeugeUeberschrift( NAME ) );
        builder.append( uebersetzer.erzeugeUeberschrift( SCHILD ) );
        builder.append( uebersetzer.erzeugeUeberschrift( TYP ) );
        builder.append( uebersetzer.erzeugeUeberschrift( BF ) );
        builder.append( uebersetzer.erzeugeUeberschrift( INI ) );
        builder.append( uebersetzer.erzeugeUeberschrift( WM ) );
        builder.append( uebersetzer.erzeugeUeberschrift( PA ) );

        for ( int i = 0 ; i < ANZAHL_SCHILDE; ++i ) {

            builder.append( gewaehlt.get( i ) );
            builder.append( name.get( i ) );
            builder.append( schild.get( i ) );
            builder.append( typ.get( i ) );
            builder.append( bf.get( i ) );
            builder.append( ini.get( i ) );
            builder.append( wm.get( i ) );
            builder.append( pa.get( i ) );
        }
        
        final JScrollPane scrollPane = 
            Util.erzeugeScrollPane( builder.getPanel() );
        
        resetter.einfuegen( scrollPane );
        
        setLayout( new BorderLayout() );
        
        add( scrollPane, BorderLayout.CENTER );
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    public final void setHeld( final Held derHeld ) {

        held = derHeld;

        held.addObserver( this );
        
        for ( final JCheckBox checkBox : gewaehlt ) {

            checkBox.setSelected( false );
        }

        final List< SchildWert > heldenschilde = held.getSchilde();

        for ( int i = 0, n = heldenschilde.size() ; i < n ; ++i ) {

            gewaehlt.get( i ).setSelected(  true );

            schilde.set( i, heldenschilde.get( i ) );
        }

        for ( int i = 0 ; i < ANZAHL_SCHILDE ; ++i ) {

            update( i );
        }
    }

    private void update( final int index ) {

        final SchildWert wert = schilde.get( index );

        bf.get( index ).setValue( wert.getBf() );

        ini.get( index ).setValue( wert.getIni() );

        name.get( index ).setText( wert.getName() );

        final String parade;
        if ( null != held ) {

            parade = String.valueOf( wert.getParade( held ) );

        } else {

            parade = "-";
        }

        pa.get( index ).setText( parade );

        schild.get( index ).removeActionListener( schildListener.get( index ) );
        schild.get( index ).setSelectedItem( wert.getSchild() );
        schild.get( index ).addActionListener( schildListener.get( index ) );

        typ.get( index ).setSelectedItem( wert.getTyp() );

        final String wmString = String.format(
                "%1$s/%2$s",
                wert.getWmAt(),
                wert.getWmPa() );

        wm.get( index ).setText( wmString );
    }

    public void update( final Observable o, final Object arg ) {
        
        if ( Datenspeicher.SCHILDE.equals( arg ) ) {
            
            final Datenspeicher speicher = Datenspeicher.getInstanz();
            
            final List< Schild > schildeListe = speicher.getSchilde();
            
            for ( final CharGenListModel< ElementMitName, Schild > model :
                schildListModels ) {
                
                model.setElements( schildeListe );
            }
        
        } else if ( o == held ) {
            
            for ( int i = 0 ; i < ANZAHL_SCHILDE; ++i ) {
                
                final SchildWert wert = schilde.get( i );
                
                final String parade = String.valueOf( wert.getParade( held ) );
                
                pa.get( i ).setText( parade );
            }
        }
    }
}
