/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.gui.steigern;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.uif_lite.panel.SimpleInternalFrame;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.gui.DialogBenutzerdefinierterVorNachteil;
import de.bernhardjung.dsaprogramm.gui.Lernmethoden;
import de.bernhardjung.dsaprogramm.gui.ResetbaresPanel;
import de.bernhardjung.dsaprogramm.util.Uebersetzbar;
import de.bernhardjung.dsaprogramm.util.Uebersetzer;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

class PanelVorNachteileSteigern 
extends ResetbaresPanel 
implements Uebersetzbar {

    /**
     * Ein Button, mit dem man für den Charakter einen Vor-/Nachteil erzeugen kann,
     * der noch nicht existiert.
     */
    private static final String EIGENER_VORNACHTEIL = "PanelVorNachteile.EigenerVorNachteil";

    /**
     * Der Titel fuer den Frame der das Panel mit den Vor- und Nachteilen
     * enthaelt die der Held besitzt. 
     */
    private static final String GEWAEHLTE = 
        "PanelVorNachteileSteigern.Gewaehlte";

    /**
     * Der Titel der Registerkarte mit den gewaehlten Vor- und Nachteilen.
     */
    private static final String GEWAEHLTE_REITER = 
        "PanelVorNachteileSteigern.Reiter.Gewaehlte";

    /**
     * Der Text bei der Combobox mit der man die Lernmethode waehlt.
     */
    private static final String LERNMETHODE =
        "PanelVorNachteileSteigern.Lernmethode";

    /**
     * Der Titel des Frames, der das Panel mit den Nachteilen enthaelt.
     */
    private static final String NACHTEILE =
        "PanelVorNachteileSteigern.Nachteile";

    /**
     * Der Titel fuer die Registerkarte die die Nachteile enthaelt.
     */
    private static final String NACHTEILE_REITER = 
        "PanelVorNachteileSteigern.Reiter.Nachteile";

    /**
     * Die Ueberschrift des Panel, das die Vor und Nachteile des Helden 
     * enthaelt.
     */
    private static final String VOR_NACHTEILE =
        "PanelVorNachteileSteigern.VorNachteile";

    /**
     * Der Titel des Frames, der das Panel mit den Vorteilen enthaelt.
     */
    private static final String VORTEILE =
        "PanelVorNachteileSteigern.Vorteile";

    /**
     * Der Titel der Registerkarte die die Vorteil anzeigt.
     */
    private static final String VORTEILE_REITER = 
        "PanelVorNachteileSteigern.Reiter.Vorteile";

    /** Die Liste der Vor-/Nachteile die der Held besitzt. */
    private final PanelVorNachteileListeSteigern gewaehlte;

    /** Die Liste der Nachteile. */
    private final PanelVorNachteileListeSteigern nachteile;


    /** Der Held, welcher im Panel bearbeitet wird. */
    private Held held;
    
    /** Uebersetzt die Texte der Benutzeroberflaeche. */
    private final Uebersetzer uebersetzer;
    
    /** Die Liste der Vorteile. */
    private final PanelVorNachteileListeSteigern vorteile;

    PanelVorNachteileSteigern() {

        uebersetzer = new Uebersetzer();
    
        
        
        final JComboBox lernmethode = uebersetzer.erzeugeComboBox(
                Lernmethode.LEHRMEISTER,
                Lernmethode.SPEZIELLE_ERFAHRUNG,
                Lernmethode.SELBSTSTUDIUM,
                Lernmethode.FREI );
        resetter.einfuegen( lernmethode );
        
        final Lernmethoden lernmethoden =
            new Lernmethoden( lernmethode );

        gewaehlte = new PanelVorNachteileListeSteigern(
                true,
                VOR_NACHTEILE,
                lernmethoden );
        resetter.einfuegen( gewaehlte );
        JButton buttonEigener = uebersetzer.erzeugeButton( EIGENER_VORNACHTEIL );
        buttonEigener.addActionListener( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                hinzufuegenEigenenVorNachteil();
            }
        } );
        JPanel buttonPanelEigener = new JPanel( new FormLayout( "4dlu, P, 4dlu", "4dlu, P, 4dlu" ) );
        buttonPanelEigener.add( buttonEigener, new CellConstraints( 2, 2 ) );
        final SimpleInternalFrame gewaehlteFrame = uebersetzer.erzeugeFrame(
                GEWAEHLTE,
                gewaehlte );

        final JPanel gewaehltePanel = new JPanel( new BorderLayout() );
        gewaehltePanel.add( buttonPanelEigener, BorderLayout.NORTH );
        gewaehltePanel.add( gewaehlteFrame );

        nachteile = new PanelVorNachteileListeSteigern(
                false,
                NACHTEILE,
                lernmethoden );
        resetter.einfuegen( nachteile );

        List< VorNachteilWert > werte =
            VorNachteilWert.erzeugeWerte( VorNachteil.getNachteile() );

        nachteile.setWerte( werte );

        vorteile = new PanelVorNachteileListeSteigern(
                false,
                VORTEILE,
                lernmethoden );
        resetter.einfuegen( vorteile );

        werte = VorNachteilWert.erzeugeWerte( VorNachteil.getVorteile() );

        vorteile.setWerte( werte );

        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        resetter.einfuegen( tabbedPane );
        
        uebersetzer.erzeugeTab( 
                tabbedPane,
                GEWAEHLTE_REITER,
                gewaehltePanel );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                VORTEILE_REITER,
                uebersetzer.erzeugeFrame( VORTEILE, vorteile ) );

        uebersetzer.erzeugeTab( 
                tabbedPane,
                NACHTEILE_REITER,
                uebersetzer.erzeugeFrame( NACHTEILE, nachteile ) );

        setLayout( new FormLayout( 
                "P:G, R:P, 2dlu, L:P, P:G", 
                "2dlu, P, 2dlu, F:D:G" ) );

        final CellConstraints cc = new CellConstraints();

        add( uebersetzer.erzeugeLabel( LERNMETHODE ), cc.xy( 2, 2 ) );

        add( lernmethode, cc.xy( 4, 2 ) );

        add( tabbedPane, cc.xyw( 1, 4, 5 ) );
    }

    private void hinzufuegenEigenenVorNachteil() {

        DialogBenutzerdefinierterVorNachteil dialog = new DialogBenutzerdefinierterVorNachteil();
        dialog.setModal( true );
        dialog.setLocationRelativeTo( this );
        dialog.setVisible( true );
        if ( dialog.isAngenommen() )
            held.einfuegen( dialog.getWert() );
    }

    public void neuUebersetzen() {
        
        uebersetzer.neuUebersetzen();
    }
    
    /**
     * @param derHeld dessen Werte angezeigt und manipuliert werden.
     */
    public final void setHeld( final Held derHeld ) {

        this.held = derHeld;

        gewaehlte.setHeld( derHeld );

        nachteile.setHeld( derHeld );

        vorteile.setHeld( derHeld );

    }

}
