/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.basis.Kulturkunde;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.util.Strings;

class VoraussetzungKulturkunde implements Voraussetzung {

    /**
     * Die Stufe die der Held in seiner Muttersprache mindestens haben muss.
     */
    private final int MINDESTSTUFE = 5;

    /** @return alle Sprachen und Klugheit. */
    public Collection<Object> getAbhaengigkeiten() {

        final List< Object > abhaengigkeiten =
            new ArrayList< Object >( Talentgruppe.SPRACHEN.size() + 1 );

        abhaengigkeiten.addAll( Talentgruppe.SPRACHEN );

        abhaengigkeiten.add( Eigenschaft.KL );

        return abhaengigkeiten;
    }

    /**
     * Prueft ob der Held eine Sprache der Kultur auf mindestens 5 beherrscht.
     */
    public KonfliktBeschreibung pruefen(
            final Held held,
            final SonderfertigkeitWert wert ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        // Die Sprache der Kultur.
        final Kulturkunde kulturkunde = 
            (Kulturkunde) wert.getVariante().getElement();

        final List< Sprache > sprachen = kulturkunde.getSprachen();

        boolean beherrschtSprache = false;

        for ( final Sprache sprache : sprachen ) {

            if ( held.hat( sprache )
                    && held.getStufe( sprache ) >= MINDESTSTUFE ) {

                beherrschtSprache = true;
                break;

            }
        }

        if ( ! beherrschtSprache ) {

            beschreibung.einfuegen( String.format(
                    I18n.i18n( "VoraussetzungKulturkunde" ),
                    Strings.erzeugeAufzaehlungMitOder( sprachen ) ) );
        }

        return beschreibung;
    }

}
