/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.talente.Talent;

class VoraussetzungSpezialisierung implements Voraussetzung {
    
    /**
     * Der Held muss das Talent auf das er sich spezialisiert auf einem 
     * vielfachen dieser Stufe beherschen. 
     */
    private static final int STUFE = 7;
    
    /**
     * @return eine leere Liste.
     */
    @SuppressWarnings( "unchecked" )
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }
    
    /**
     * Prueft ob der Held das Talent auf einer entsprechend hohen Stufe besitzt. 
     */
    public KonfliktBeschreibung pruefen(
            final Held held, 
            final SonderfertigkeitWert wert ) {
        
        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        final Talent talent = wert.getSonderfertigkeit().getTalent();
        
        final int index = held.getIndex( wert );
        
        final int mindeststufe = STUFE * index;
        
        if ( ! held.hat( talent )
                || held.getStufe( talent ) < mindeststufe ) {
            
            beschreibung.einfuegen( String.format( 
                    I18n.i18n( "VoraussetzungSpezialisierung" ), 
                    talent.getName(),
                    mindeststufe ) );
            
        }
        
        return beschreibung;
    }

}
