/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.kosten;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.basis.varianten.Variante;
import de.bernhardjung.dsaprogramm.basis.varianten.VariantenTalentgruppe;
import de.bernhardjung.dsaprogramm.talente.Kampftechnik;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class KostenKampftechnik extends Kosten {

    private VorNachteilWert begabungFernkampf;

    private VorNachteilWert begabungKampftechniken;

    private VorNachteilWert begabungNahkampf;

    private VorNachteilWert begabungTalent;

    private VorNachteilWert unfaehigkeitFernkampf;

    private VorNachteilWert unfaehigkeitNahkampf;

    /** @return Begabung Talent, Begabung Talentgruppe, Elfische Weltsicht. */
    @Override
    public Collection<Object> getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = super.getAbhaengigkeiten();

        abhaengigkeiten.add( VorNachteil.AKADEMISCHE_AUSBILDUNG_KRIEGER );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENT );

        abhaengigkeiten.add( VorNachteil.BEGABUNG_TALENTGRUPPE );

        abhaengigkeiten.add( VorNachteil.ELFISCHE_WELTSICHT );

        abhaengigkeiten.add( VorNachteil.SCHLANGENMENSCH );

        return abhaengigkeiten;
    }

    /** {@inheritDoc} */
    @Override
    public  int getSteigerungsspalte(
            final Held held,
            final Talent talent,
            final int stufe,
            final Lernmethode lernmethode ) {

        int steigerungsfaktor =
            super.getSteigerungsspalte( held, talent, stufe, lernmethode );

        setBegabungUnfaehigkeit( talent );

        final Kampftechnik kampftechnik = (Kampftechnik) talent;

        if ( ( kampftechnik.istFernkampfTechnik()
                && held.hat( unfaehigkeitFernkampf ) )

          || ( kampftechnik.istNahkampfTechnik()
                && held.hat( unfaehigkeitNahkampf ) ) ) {

            ++steigerungsfaktor;
        }

        boolean begabungWirkt = false;

        if ( held.wirdErschaffen()
                && ( stufe < 10 )
                && held.hat( VorNachteil.AKADEMISCHE_AUSBILDUNG_KRIEGER ) ) {

            steigerungsfaktor -= 2;

        } else {

            if ( held.hat( begabungTalent ) ) {

                --steigerungsfaktor;

                begabungWirkt = true;
            }

            if ( held.hat( begabungKampftechniken )

              || ( held.hat( begabungNahkampf )
                    && kampftechnik.istNahkampfTechnik() )

              || ( held.hat( begabungFernkampf )
                    && kampftechnik.istFernkampfTechnik() ) ) {

                --steigerungsfaktor;

                begabungWirkt = true;
            }

        }

        if ( ( ! begabungWirkt )
          && ( Talent.RINGEN == talent )
          && held.hat( VorNachteil.SCHLANGENMENSCH ) ) {

            --steigerungsfaktor;
        }

        if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
          && ( ! held.hatLeittalent( talent ) ) ) {

            ++steigerungsfaktor;
        }

        return steigerungsfaktor;
    }

    /**
     * Initialisiert die Begabungen und Unfaehigkeiten und
     * setzt die Variante von {@link #begabungTalent} auf das uebergebene
     * Talent.
     *
     * @param talent
     *            fuer die Begabung Talent.
     */
    private void setBegabungUnfaehigkeit( final Talent talent ) {

        if ( null == begabungTalent ) {

            begabungTalent = new VorNachteilWert( 
                    VorNachteil.BEGABUNG_TALENT,
                    new Variante( talent ) );

            begabungFernkampf = new VorNachteilWert( 
                    VorNachteil.BEGABUNG_TALENTGRUPPE,
                    new Variante( VariantenTalentgruppe.FERNKAMPF ) );

            begabungKampftechniken = new VorNachteilWert( 
                    VorNachteil.BEGABUNG_TALENTGRUPPE,
                    new Variante( Talentgruppe.KAMPF ) );
            
            begabungNahkampf = new VorNachteilWert( 
                    VorNachteil.BEGABUNG_TALENTGRUPPE,
                    new Variante( VariantenTalentgruppe.NAHKAMPF ) );

            unfaehigkeitFernkampf = new VorNachteilWert( 
                    VorNachteil.UNFAEHIGKEIT_TALENTGRUPPE,
                    new Variante( VariantenTalentgruppe.FERNKAMPF ) );

            unfaehigkeitNahkampf = new VorNachteilWert( 
                    VorNachteil.UNFAEHIGKEIT_TALENTGRUPPE,
                    new Variante( VariantenTalentgruppe.NAHKAMPF ) );

        } else {

            begabungTalent.setVariante( talent );

        }
    }
}
