/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.talente.voraussetzungen;

import java.util.Collection;
import java.util.Collections;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.KonfliktBeschreibung;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;

class VoraussetzungStandard extends Voraussetzung {

    /**
     * @return leere Liste.
     */
    @Override
    @SuppressWarnings("unchecked")
    public Collection<Object> getAbhaengigkeiten() {
        return Collections.EMPTY_LIST;
    }

    /** {@inheritDoc} */
    @Override
    public KonfliktBeschreibung pruefen(
            final Held held,
            final Talent talent ) {

        final KonfliktBeschreibung beschreibung = new KonfliktBeschreibung();

        if ( held.hat( talent ) ) {

            // Grenzwerte testen
            final int mindeststufe;
            if ( held.hatVorgegeben( talent ) ) {

                mindeststufe = held.getMindeststufe( talent );

            } else {

                mindeststufe = 0;

            }

            final int hoechststufe = talent.getHoechststufe( held );

            final int stufe = held.getStufe( talent );

            if ( ( stufe < mindeststufe ) || ( hoechststufe < stufe ) ) {

                beschreibung.einfuegen( String.format( 
                        I18n.i18n( "VoraussetzungStandard.Talent.Grenzen" ), 
                        mindeststufe, 
                        hoechststufe ) );
            }


        } else {

            // Der Held besitzt das Talent nicht. Deswegen sind wir in diesem
            // else Teil gelandet ...
            if ( held.hatVorgegeben( talent ) ) {
                // ... er muss es aber besitzen
                beschreibung.einfuegen( 
                        I18n.i18n( "VoraussetzungStandard.Talent.Vorgegeben" ) );
            }
        }

        return beschreibung;
    }

}
