/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.anpasser;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.TalentWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

class AnpasserTalent implements Anpasser {

    /** Das Talent das der Held erhalten/verlieren soll. */
    private final Talent talent;

    /**
     * @param dasTalent das der Held erhalten/verlieren soll.
     */
    AnpasserTalent( final Talent dasTalent ) {

        talent = dasTalent;
    }

    /**
     *  Aktiviert ein Talent auf Stufe 3.
     */
    public void eingefuegt( final Held held, final VorNachteilWert wert ) {

        if ( held.wirdErschaffen() ) {

            if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT ) ) {

                held.einfuegenLeittalent( talent );
            }

            held.einfuegenVorgegeben( new TalentWert( talent, 3 ) );
        }
    }

    /**
     * Deaktiviert das Talent.
     */
    public void entfernt( final Held held, final VorNachteilWert wert ) {

        if ( held.wirdErschaffen() ) {

            held.entfernenVorgegeben( talent );
        }
    }

    /** Macht nichts. */
    public void stufeGeaendert(final Held held, final VorNachteilWert wert) {
        // kein Implementierung noetig.
    }

    public void geaendertVariante(
            final Held held,
            final Object alteVariante,
            final Object neueVariante ) {
        // nichts zu tun.
    }

}
