/*
 * This file is part of the TDE Character Generator (TCG).
 *
 * TCG is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * TCG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TCG.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.bernhardjung.dsaprogramm.vornachteile.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

import java.util.Arrays;
import java.util.Collection;

class KostenAlbino implements Kosten {

    /** Die normalen GP-Kosten des Nachteils Albino. */
    static final int BASISKOSTEN = -7;

    /**
     * Der SO Wert ueber dem jeder Punkt mit einem GP extra bezahlt werden
     * muss.
     */
    static final int SO_GRENZE = 7;

    /** {@inheritDoc} */
    public int getAbbaukosten(
            final Held held,
            final VorNachteilWert wert,
            final Lernmethode lernmethode ) {
        if ( lernmethode == Lernmethode.FREI )
            return 0;
        else
            return -100 * BASISKOSTEN;
    }

    /** @return {@link Eigenschaft#SO} */
    public Collection< Object > getAbhaengigkeiten() {
        return Arrays.asList( (Object) Eigenschaft.SO );
    }

    /** @return die Kosten von Albino. */
    public int getGpKosten() {
        return BASISKOSTEN;
    }

    /** @return die Kosten von Albino. */
    public int getGpKosten( final VorNachteilWert wert ) {
        return BASISKOSTEN;
    }

    /**
     * @return die Kosten von Albino, wobei die Kosten fuer jeden Punkte SO
     *         ueber 7 um einen Punkte erhoeht werden.
     */
    public int getGpKosten( final Held held, final VorNachteilWert wert ) {

        final int kosten = BASISKOSTEN;

        final int so = held.getStufe( Eigenschaft.SO );

        // das sind die Punkte ueber 7, die extra zu bezahlen sind.
        int extraPunkte = so - SO_GRENZE;

        // Wenn extraPunkte kleiner 0 ist, muessen keine Punkte bezahlt werden.
        extraPunkte = Math.max( 0, extraPunkte );

        return kosten + extraPunkte;
    }

    /** @return 0 */
    public int getSenkungskosten(final Lernmethode lernmethode, final VorNachteilWert wert ) {
        return 0;
    }

    /** @return {@code false} */
    public final boolean istSenkbar( final VorNachteilWert wert ) {
        return false;
    }
}
